# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest
import sys
import os
import json
from unittest.mock import patch, MagicMock

from gns3.local_config import LocalConfig
from gns3.version import __version_info__


@pytest.fixture
def config_file(tmpdir):
    with open(str(tmpdir / "test.cfg"), "w+") as f:
        f.write(json.dumps({
            "VirtualBox": {
                "use_local_server": True,
                "invalid_key": False,
            },
            "VPCS": {
                "vpcs_path": "/usr/local/bin/vpcs"
            },
            "type": "settings",
            "version": "1.4.0.dev1"}))
    return str(tmpdir / "test.cfg")


def test_loadSectionSettingsEmptyFile(local_config):

    assert local_config.loadSectionSettings("Test", {}) == {}
    assert local_config.loadSectionSettings("Test2", {"a": "b"}) == {"a": "b"}
    assert local_config.loadSectionSettings("Test3", {"a": {"b": 1}}) == {"a": {"b": 1}}


def test_loadSectionSettingsPartialConfig(local_config):

    local_config._settings = {
        "Test": {"a": "b"},
        "Test2": {"a": "c"},
        "Test3": {"a": {"b": 1}},
        "Servers": {
            "local_server": {"user": "root"},
            "remote_servers": [
                {"z": "x"}
            ]
        }
    }

    assert local_config.loadSectionSettings("Test", {}) == {"a": "b"}
    assert local_config.loadSectionSettings("Test2", {"a": "b"}) == {"a": "c"}
    assert local_config.loadSectionSettings("Test3", {"a": {"b": 1}}) == {"a": {"b": 1}}
    assert local_config.loadSectionSettings("Servers", {
        "local_server": {"user": "", "password": ""},
        "remote_servers": []
    }) == {
        "local_server": {"user": "root", "password": ""},
        "remote_servers": [
            {"z": "x"}
        ]
    }


def test_readConfig(config_file, local_config):
    local_config.setConfigFilePath(config_file)
    assert local_config.configFilePath() == config_file

    section = local_config.loadSectionSettings("VirtualBox", {
        "use_local_server": False
    })
    assert section["use_local_server"]
    assert "invalid_key" in section


def test_readConfigReload(config_file, tmpdir, local_config):

    local_config.setConfigFilePath(config_file)
    assert local_config.configFilePath() == config_file

    section = local_config.loadSectionSettings("VirtualBox", {
        "use_local_server": False
    })

    assert section["use_local_server"]

    with open(str(tmpdir / "test.cfg"), "w+") as f:
        f.write(json.dumps({
            "VirtualBox": {
                "use_local_server": False
            },
            "type": "settings",
            "version": "1.4.0.dev1"}))

    local_config._readConfig(config_file)
    assert local_config._settings["VirtualBox"]["use_local_server"] is False


@pytest.mark.skipif(sys.platform.startswith('darwin') is False, reason='Only on MacOS')
def test_migrateOldConfigOSX(tmpdir):

    with patch('os.path.expanduser', return_value=str(tmpdir)):

        os.makedirs(str(tmpdir / '.config' / 'gns3.net'))
        open(str(tmpdir / '.config' / 'gns3.net' / 'hello'), 'w+').close()

        local_config = LocalConfig()

        version = "{}.{}".format(__version_info__[0], __version_info__[1])
        assert os.path.exists(str(tmpdir / '.config' / 'GNS3' / version))
        assert os.path.exists(str(tmpdir / '.config' / 'GNS3' / version / 'hello'))
        assert os.path.exists(str(tmpdir / '.config' / 'gns3.net'))

        # It should migrate only one time
        open(str(tmpdir / '.config' / 'gns3.net' / 'world'), 'w+').close()

        local_config._migrateOldConfigPath()
        assert os.path.exists(str(tmpdir / '.config' / 'GNS3' / version / 'hello'))
        assert not os.path.exists(str(tmpdir / '.config' / 'GNS3' / version / 'world'))


def test_migrate13Config(tmpdir):
    config_file = str(tmpdir / "gns3_gui.conf")
    local_config = LocalConfig(config_file=config_file)

    server_config = {
        "allow_console_from_anywhere": True,
        "auth": False,
        "auto_start": True,
        "console_end_port_range": 5000,
        "console_start_port_range": 2001,
        "host": "127.0.0.1",
        "images_path": "/home/gns3/GNS3/images",
        "password": "",
        "path": "/bin/gns3server",
        "port": 8001,
        "projects_path": "/home/gns3/GNS3/projects",
        "report_errors": False,
        "udp_end_port_range": 20000,
        "udp_start_port_range": 10000,
        "user": ""
    }

    with open(config_file, "w+") as f:
        f.write(json.dumps({
            "LocalServer": server_config,
            "RemoteServers": [
                server_config
            ],
            "GUI": {
                "hide_getting_started_dialog": True
            },
            "type": "settings",
            "version": "1.3.7"}))

    local_config._readConfig(config_file)
    local_config._migrateOldConfig()

    # The old config should not be erased in order to avoid losing data when rollback to 1.3
    assert local_config._settings["LocalServer"] == server_config
    assert local_config._settings["RemoteServers"] == [server_config]
    assert local_config._settings["Servers"]["local_server"] == server_config
    assert local_config._settings["Servers"]["remote_servers"] == [server_config]
    assert local_config._settings["MainWindow"]["hide_getting_started_dialog"]

    # When the file is migrated to 1.4 we should not try to modify it
    with open(config_file, "w+") as f:
        f.write(json.dumps({
            "LocalServer": {"host": "error"},
            "type": "settings",
            "version": "1.4.2"}))

    local_config._readConfig(config_file)
    local_config._migrateOldConfig()

    assert local_config._settings["LocalServer"]["host"] == "error"
    assert local_config._settings["Servers"]["local_server"]["host"] == "127.0.0.1"


def test_migrate13ConfigOldOsxServerPath(tmpdir):
    config_file = str(tmpdir / "gns3_gui.conf")
    local_config = LocalConfig(config_file=config_file)

    server_config = {
        "allow_console_from_anywhere": True,
        "auth": False,
        "auto_start": True,
        "console_end_port_range": 5000,
        "console_start_port_range": 2001,
        "host": "127.0.0.1",
        "images_path": "/home/gns3/GNS3/images",
        "password": "",
        "path": "/Applications/GNS3.app/Contents/Resources/server/Contents/MacOS/gns3server",
        "port": 8001,
        "projects_path": "/home/gns3/GNS3/projects",
        "report_errors": False,
        "udp_end_port_range": 20000,
        "udp_start_port_range": 10000,
        "user": ""
    }

    with open(config_file, "w+") as f:
        f.write(json.dumps({
            "LocalServer": server_config,
            "RemoteServers": [
                server_config
            ],
            "type": "settings",
            "version": "1.3.7"}))

    local_config._readConfig(config_file)
    local_config._migrateOldConfig()

    # The old config should not be erased in order to avoid losing data when rollback to 1.3
    assert local_config._settings["LocalServer"]["path"] == "/Applications/GNS3.app/Contents/Resources/server/Contents/MacOS/gns3server"
    assert local_config._settings["Servers"]["local_server"]["path"] == "gns3server"


def test_isMainGui_pid_file_not_exist(tmpdir):
    with patch("gns3.local_config.LocalConfig.configDirectory") as mock_config_directory:
        mock_config_directory.return_value = str(tmpdir)
        assert LocalConfig().isMainGui() is True
        assert os.path.exists(str(tmpdir / "gns3_gui.pid"))


def test_isMainGui_pid_file_exist_but_different(tmpdir):
    with open(str(tmpdir / "gns3_gui.pid"), "w+") as f:
        f.write("42")

    mock_process = MagicMock()
    mock_process.name.return_value = "gns3.exe"
    if not sys.platform.startswith("win"):
        mock_process.uids.return_value = (os.getuid(), os.getuid(), os.getuid())
    with patch("psutil.Process", return_value=mock_process):
        with patch("gns3.local_config.LocalConfig.configDirectory") as mock_config_directory:
            mock_config_directory.return_value = str(tmpdir)
            assert LocalConfig().isMainGui() is False


def test_isMainGui_pid_file_exist_but_different_proces_dead(tmpdir):
    with open(str(tmpdir / "gns3_gui.pid"), "w+") as f:
        f.write("42")

    mock_process = MagicMock()
    with patch("psutil.Process", return_value=mock_process) as mock:
        mock.name.side_effect = (lambda: exec('raise(OSError())'))
        with patch("gns3.local_config.LocalConfig.configDirectory") as mock_config_directory:
            mock_config_directory.return_value = str(tmpdir)
            assert LocalConfig().isMainGui() is True


def test_isMainGui_pid_file_exist_but_same_pid(tmpdir):
    with open(str(tmpdir / "gns3_gui.pid"), "w+") as f:
        f.write("42")

    with patch("os.getpid", return_value=42):
        with patch("gns3.local_config.LocalConfig.configDirectory") as mock_config_directory:
            mock_config_directory.return_value = str(tmpdir)
            assert LocalConfig().isMainGui() is True


def test_migrateRemoveInternetVM(tmpdir):
    """
    In 2.0 the internet VM is replaced by the nat node
    """
    config_file = str(tmpdir / "gns3_gui.conf")
    local_config = LocalConfig(config_file=config_file)

    with open(config_file, "w+") as f:
        f.write(json.dumps({
            "Qemu": {
                "use_local_server": True,
                "enable_kvm": True,
                "vms": [
                    {
                        "kernel_image": "",
                        "kernel_command_line": "",
                        "first_port_name": "",
                        "cpus": 1,
                        "process_priority": "normal",
                        "port_name_format": "Ethernet{0}",
                        "port_segment_size": 0,
                        "default_name_format": "{name}-{0}",
                        "mac_address": "",
                        "usage": "",
                        "qemu_path": "/usr/local/bin/qemu-system-x86_64",
                        "ram": 256,
                        "symbol": ":/symbols/qemu_guest.svg",
                        "category": 2,
                        "server": "local",
                        "console_type": "telnet",
                        "hdc_disk_interface": "ide",
                        "hda_disk_image": "linux-microcore-6.4.img",
                        "initrd": "",
                        "hda_disk_interface": "ide",
                        "cpu_throttling": 0,
                        "adapter_type": "e1000",
                        "options": "-nographic",
                        "acpi_shutdown": False,
                        "boot_priority": "c",
                        "hdb_disk_interface": "ide",
                        "name": "dsfgsdf",
                        "hdd_disk_image": "",
                        "adapters": 1,
                        "cdrom_image": "",
                        "hdb_disk_image": "",
                        "linked_clone": True,
                        "hdd_disk_interface": "ide",
                        "hdc_disk_image": "",
                        "platform": ""
                    },
                    {
                        "kernel_image": "",
                        "kernel_command_line": "",
                        "server": "vm",
                        "process_priority": "normal",
                        "qemu_path": "/usr/bin/qemu-system-i386",
                        "name": "Internet 0.1",
                        "symbol": ":/symbols/cloud.svg",
                        "category": 0,
                        "usage": "Just connect stuff to the appliance. Everything is automated.",
                        "console_type": "telnet",
                        "initrd": "",
                        "hda_disk_image": "core-linux-6.4-internet-0.1.img",
                        "hdc_disk_interface": "ide",
                        "hda_disk_interface": "ide",
                        "cpu_throttling": 0,
                        "adapter_type": "e1000",
                        "options": "-device e1000,netdev=internet0 -netdev vde,sock=/var/run/vde2/qemu0.ctl,id=internet0 -nographic",
                        "hdb_disk_interface": "ide",
                        "hdd_disk_image": "",
                        "adapters": 1,
                        "cdrom_image": "",
                        "hdb_disk_image": "",
                        "hdd_disk_interface": "ide",
                        "hdc_disk_image": "",
                        "ram": 64
                    }
                ]
            },
            "type": "settings",
            "version": "1.5.2"}))

    local_config._readConfig(config_file)
    local_config._migrateOldConfig()

    assert len(local_config._settings["Qemu"]["vms"]) == 1


def test_runAsRootPath(local_config):
    assert 'run_as_root' in local_config.runAsRootPath()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for server preferences.
"""

import os
import sys
import copy
import shutil

import logging
log = logging.getLogger(__name__)

from gns3.qt import QtNetwork, QtWidgets
from ..ui.controller_preferences_page_ui import Ui_ControllerPreferencesPageWidget
from ..topology import Topology
from ..settings import CONTROLLER_SETTINGS, DEFAULT_CONTROLLER_HOST
from ..dialogs.edit_compute_dialog import EditComputeDialog
from ..local_server import LocalServer
from ..compute_manager import ComputeManager


class ControllerPreferencesPage(QtWidgets.QWidget, Ui_ControllerPreferencesPageWidget):

    """
    QWidget configuration page for controller preferences.
    """

    def __init__(self, parent=None):

        super().__init__()
        self.setupUi(self)
        self._remote_computes = {}

        # connect the slots
        self.uiLocalServerToolButton.clicked.connect(self._localServerBrowserSlot)
        self.uiUbridgeToolButton.clicked.connect(self._ubridgeBrowserSlot)
        self.uiAddRemoteServerPushButton.clicked.connect(self._remoteServerAddSlot)
        self.uiDeleteRemoteServerPushButton.clicked.connect(self._remoteServerDeleteSlot)
        self.uiUpdateRemoteServerPushButton.clicked.connect(self._remoteServerUpdateSlot)

        self.uiRemoteServersTreeWidget.itemSelectionChanged.connect(self._remoteServerChangedSlot)
        self.uiRestoreDefaultsPushButton.clicked.connect(self._restoreDefaultsSlot)
        self.uiLocalServerAutoStartCheckBox.stateChanged.connect(self._useLocalServerAutoStartSlot)

        # load all available addresses
        for address in QtNetwork.QNetworkInterface.allAddresses():
            if address.protocol() in [QtNetwork.QAbstractSocket.IPv4Protocol, QtNetwork.QAbstractSocket.IPv6Protocol]:
                address_string = address.toString()
                if address_string.startswith("169.254") or address_string.startswith("fe80"):
                    # ignore link-local addresses
                    continue
                self.uiLocalServerHostComboBox.addItem(address_string, address_string)
        self.uiLocalServerHostComboBox.addItem("localhost", "localhost")  # local host
        self.uiLocalServerHostComboBox.addItem("::", "::")  # all IPv6 addresses
        self.uiLocalServerHostComboBox.addItem("0.0.0.0", "0.0.0.0")  # all IPv4 addresses

        # default is 127.0.0.1
        index = self.uiLocalServerHostComboBox.findText(DEFAULT_CONTROLLER_HOST)
        if index != -1:
            self.uiLocalServerHostComboBox.setCurrentIndex(index)

    def _useLocalServerAutoStartSlot(self, state):
        """
        Slot to enable or not local server settings.
        """

        if state:
            self.uiGeneralSettingsGroupBox.setVisible(True)
            self.uiConsolePortRangeGroupBox.setVisible(True)
            self.uiUDPPortRangeGroupBox.setVisible(True)
            self.uiRemoteMainServerGroupBox.setVisible(False)
        else:
            self.uiRemoteMainServerGroupBox.setVisible(True)
            self.uiGeneralSettingsGroupBox.setVisible(False)
            self.uiConsolePortRangeGroupBox.setVisible(False)
            self.uiUDPPortRangeGroupBox.setVisible(False)

    def _restoreDefaultsSlot(self):
        """
        Slot to restore default settings
        """

        self._populateWidgets(CONTROLLER_SETTINGS)

    def _localServerBrowserSlot(self):
        """
        Slot to open a file browser and select a local server.
        """

        filter = ""
        if sys.platform.startswith("win"):
            filter = "Executable (*.exe);;All files (*.*)"
        server_path = shutil.which("gns3server")
        path, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Select the local server", server_path, filter)
        if not path:
            return

        self.uiLocalServerPathLineEdit.setText(path)

    def _ubridgeBrowserSlot(self):
        """
        Slot to open a file browser and select the ubridge executable path.
        """

        filter = ""
        if sys.platform.startswith("win"):
            filter = "Executable (*.exe);;All files (*.*)"

        ubridge_path = shutil.which("ubridge")
        path, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Select ubridge executable", ubridge_path, filter)
        if not path:
            return

        self.uiUbridgePathLineEdit.setText(path)

    def _remoteServerChangedSlot(self):
        """
        Enables the use of the delete button.
        """

        item = self.uiRemoteServersTreeWidget.currentItem()
        if item:
            self.uiDeleteRemoteServerPushButton.setEnabled(True)
            self.uiUpdateRemoteServerPushButton.setEnabled(True)
        else:
            self.uiDeleteRemoteServerPushButton.setEnabled(False)
            self.uiUpdateRemoteServerPushButton.setEnabled(False)

    def _remoteServerAddSlot(self):
        """
        Adds a new remote server.
        """

        dialog = EditComputeDialog(self.parent())
        dialog.show()
        if dialog.exec_():
            self._remote_computes[dialog.compute().id()] = dialog.compute()
            self._populateRemoteServersTree()

    def _remoteServerDeleteSlot(self):
        """
        Deletes a remote server.
        """

        item = self.uiRemoteServersTreeWidget.currentItem()
        if item:
            del self._remote_computes[item.compute_id]
            self.uiRemoteServersTreeWidget.takeTopLevelItem(self.uiRemoteServersTreeWidget.indexOfTopLevelItem(item))

    def _remoteServerUpdateSlot(self):
        """
        Update a remote server.
        """

        item = self.uiRemoteServersTreeWidget.currentItem()
        dialog = EditComputeDialog(self.parent(), item.compute)
        dialog.show()
        if dialog.exec_():
            self._populateRemoteServersTree()

    def _populateWidgets(self, servers_settings):
        """
        Populates the widgets with the settings.

        :param servers_settings: servers settings
        """

        self.uiLocalServerPathLineEdit.setText(servers_settings["path"])
        self.uiUbridgePathLineEdit.setText(servers_settings["ubridge_path"])
        index = self.uiLocalServerHostComboBox.findData(servers_settings["host"])
        if index != -1:
            self.uiLocalServerHostComboBox.setCurrentIndex(index)
        self.uiLocalServerPortSpinBox.setValue(servers_settings["port"])

        self.uiRemoteMainServerHostLineEdit.setText(servers_settings["host"])
        self.uiRemoteMainServerPortSpinBox.setValue(servers_settings["port"])
        self.uiRemoteMainServerProtocolComboBox.setCurrentText(servers_settings["protocol"].upper())
        self.uiRemoteMainServerUserLineEdit.setText(servers_settings["username"])
        self.uiRemoteMainServerPasswordLineEdit.setText(servers_settings["password"])

        #self.uiLocalServerAutoStartCheckBox.setChecked(servers_settings["auto_start"])
        #self._useLocalServerAutoStartSlot(servers_settings["auto_start"])

        # FIXME: only allow remote server (temporary)
        self.uiLocalServerAutoStartCheckBox.setChecked(False)
        self.uiLocalServerAutoStartCheckBox.setEnabled(False)
        self._useLocalServerAutoStartSlot(False)

        self.uiConsoleConnectionsToAnyIPCheckBox.setChecked(servers_settings["allow_console_from_anywhere"])
        self.uiDynamicComputeAllocationCheckBox.setChecked(servers_settings["dynamic_compute_allocation"])
        self.uiConsoleStartPortSpinBox.setValue(servers_settings["console_start_port_range"])
        self.uiConsoleEndPortSpinBox.setValue(servers_settings["console_end_port_range"])
        self.uiUDPStartPortSpinBox.setValue(servers_settings["udp_start_port_range"])
        self.uiUDPEndPortSpinBox.setValue(servers_settings["udp_end_port_range"])

    def _populateRemoteServersTree(self):
        self.uiRemoteServersTreeWidget.clear()
        for compute in self._remote_computes.values():
            item = QtWidgets.QTreeWidgetItem(self.uiRemoteServersTreeWidget)
            item.setText(0, compute.name())
            item.setText(1, compute.protocol())
            item.setText(2, compute.host())
            item.setText(3, str(compute.port()))
            item.setText(4, compute.user())
            item.compute = self._remote_computes[compute.id()]
            item.compute_id = compute.id()
        self.uiRemoteServersTreeWidget.resizeColumnToContents(0)
        self._remoteServerChangedSlot()

    def loadPreferences(self):
        """
        Loads the server preferences.
        """

        # Settings from the gns3_server.conf
        local_server_settings = LocalServer.instance().localServerSettings()
        self._populateWidgets(local_server_settings)

        cm = ComputeManager.instance()
        # load remote server preferences
        self._remote_computes.clear()
        for compute in cm.remoteComputes():
            # We copy to be able to detect the change with the original element
            # when we apply the settings
            self._remote_computes[compute.id()] = copy.copy(compute)
        self._populateRemoteServersTree()

    def savePreferences(self):
        """
        Saves the server preferences.
        """

        local_server_settings = LocalServer.instance().localServerSettings()

        # save the local server preferences
        new_local_server_settings = local_server_settings.copy()
        new_local_server_settings.update({"path": self.uiLocalServerPathLineEdit.text(),
                                          "ubridge_path": self.uiUbridgePathLineEdit.text(),
                                          "host": self.uiLocalServerHostComboBox.itemData(self.uiLocalServerHostComboBox.currentIndex()),
                                          "port": self.uiLocalServerPortSpinBox.value(),
                                          "auto_start": self.uiLocalServerAutoStartCheckBox.isChecked(),
                                          "allow_console_from_anywhere": self.uiConsoleConnectionsToAnyIPCheckBox.isChecked(),
                                          "dynamic_compute_allocation": self.uiDynamicComputeAllocationCheckBox.isChecked(),
                                          "console_start_port_range": self.uiConsoleStartPortSpinBox.value(),
                                          "console_end_port_range": self.uiConsoleEndPortSpinBox.value(),
                                          "udp_start_port_range": self.uiUDPStartPortSpinBox.value(),
                                          "udp_end_port_range": self.uiUDPEndPortSpinBox.value()})

        if new_local_server_settings["console_end_port_range"] <= new_local_server_settings["console_start_port_range"]:
            QtWidgets.QMessageBox.critical(self, "Port range", "Invalid console port range from {} to {}".format(new_local_server_settings["console_start_port_range"],
                                                                                                                 new_local_server_settings["console_end_port_range"]))
            return

        if new_local_server_settings["udp_end_port_range"] <= new_local_server_settings["udp_start_port_range"]:
            QtWidgets.QMessageBox.critical(self, "Port range", "Invalid UDP port range from {} to {}".format(new_local_server_settings["udp_start_port_range"],
                                                                                                             new_local_server_settings["udp_end_port_range"]))
            return

        if new_local_server_settings["auto_start"]:
            if not os.path.isfile(new_local_server_settings["path"]):
                QtWidgets.QMessageBox.critical(self, "Local server", "Could not find local server {}".format(new_local_server_settings["path"]))
                return
            if not os.access(new_local_server_settings["path"], os.X_OK):
                QtWidgets.QMessageBox.critical(self, "Local server", "{} is not an executable".format(new_local_server_settings["path"]))

            if new_local_server_settings != local_server_settings:
                # first check if we have nodes on the local server
                topology = Topology.instance()
                if len(topology.nodes()):
                    QtWidgets.QMessageBox.critical(self, "Local server", "Please close your project or delete all the nodes running on the local server before changing the local server settings")
                    return
                LocalServer.instance().updateLocalServerSettings(new_local_server_settings)
        else:
            new_local_server_settings["host"] = self.uiRemoteMainServerHostLineEdit.text()
            new_local_server_settings["port"] = self.uiRemoteMainServerPortSpinBox.value()
            new_local_server_settings["protocol"] = self.uiRemoteMainServerProtocolComboBox.currentText().lower()
            new_local_server_settings["username"] = self.uiRemoteMainServerUserLineEdit.text()
            new_local_server_settings["password"] = self.uiRemoteMainServerPasswordLineEdit.text()

            # Some users get confused by compute and controller and
            # configure the same server twice
            for compute in self._remote_computes.values():
                if new_local_server_settings["host"] == compute.host() and new_local_server_settings["port"] == compute.port():
                    QtWidgets.QMessageBox.critical(self, "Remote compute", "Is it not possible to use a controller as a remote compute")
                    return
            LocalServer.instance().updateLocalServerSettings(new_local_server_settings)

        ComputeManager.instance().updateList(self._remote_computes.values())
        self.loadPreferences()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from .qt import sip
import struct
import inspect
import datetime
import platform

from .qt import QtCore, QtWidgets
from .topology import Topology
from .version import __version__
from .console_cmd import ConsoleCmd
from .pycutext import PyCutExt
from .modules import MODULES
from .local_config import LocalConfig

import logging
log = logging.getLogger(__name__)


class ConsoleLogHandler(logging.StreamHandler):
    """
    Display log event to the console
    """

    def emit(self, record):
        if sip.isdeleted(self._console_view):
            return

        message = self.format(record)
        level_no = record.levelno
        if level_no >= logging.ERROR:
            self._console_view.write_message_signal.emit("{}\n".format(message), "error")
        elif level_no >= logging.WARNING:
            self._console_view.write_message_signal.emit("{}\n".format(message), "warning")
        elif level_no >= logging.INFO:
            # To avoid noise on console we display all event only if log level is debug
            # or if we force the display in the log record
            if "show" in record.__dict__ or logging.getLogger().getEffectiveLevel() == logging.DEBUG:
                self._console_view.write_message_signal.emit("{}\n".format(message), "debug")
        elif level_no >= logging.DEBUG:
            self._console_view.write_message_signal.emit("{}\n".format(message), "debug")


class ConsoleView(PyCutExt, ConsoleCmd):

    # Emit this signal to write a message on console
    write_message_signal = QtCore.Signal(str, str)

    def __init__(self, parent):

        # Set the prompt PyCutExt
        self.prompt = '=> '
        sys.ps1 = '=> '

        # Set introduction message
        bitness = struct.calcsize("P") * 8
        current_year = datetime.date.today().year
        self.intro = "GNS3 management console.\nRunning GNS3 version {} on {} ({}-bit) with Python {} Qt {} and PyQt {}.\n" \
                     "Copyright (c) 2006-{} GNS3 Technologies.\n" \
                     "Use Help -> GNS3 Doctor to detect common issues." \
                     "".format(__version__, platform.system(), bitness, platform.python_version(), QtCore.QT_VERSION_STR, QtCore.PYQT_VERSION_STR, current_year)

        # Parent class initialization
        try:
            super().__init__(None, self.intro, parent=parent)

            # dynamically get all the available commands so we can color them
            methods = inspect.getmembers(self, predicate=inspect.ismethod)
            commands = []
            for method in methods:
                method_name = method[0]
                if method_name.startswith("do_"):
                    commands.append(method_name[3:])

            # put our own keywords list
            self.colorizer.keywords = commands
        except Exception as e:
            sys.stderr.write(e)

        self._handleLogs()

        if LocalConfig.instance().experimental():
            log.warning("WARNING: Experimental features enable. You can use some unfinished features and lost data.")

        for module in MODULES:
            instance = module.instance()
            instance.notification_signal.connect(self.writeNotification)

        self.write_message_signal.connect(self._writeMessageSlot)

        # required for Cmd module (do_help etc.)
        self.stdout = sys.stdout
        self._topology = Topology.instance()

    def contextMenuEvent(self, event):
        """
        Handles all context menu events.

        :param event: QContextMenuEvent instance
        """

        menu = self.createStandardContextMenu()
        delete_all_action = QtWidgets.QAction("Delete All", menu)
        delete_all_action.triggered.connect(self._deleteAllActionSlot)
        menu.addAction(delete_all_action)
        menu.exec_(event.globalPos());

    def _deleteAllActionSlot(self):
        """
        Delete all action slot
        """

        self.clear()
        self.write(self.prompt)
        self.lines = []
        self._clearLine()

    def _writeMessageSlot(self, message, level):
        """
        Write a message in the console.
        """
        if level == "error":
            self.write(message, error=True)
        elif level == "warning":
            self.write(message, warning=True)
        else:
            self.write(message)

    def _handleLogs(self):
        """
        Catch log message and display them
        """

        log = logging.getLogger()
        log_handler = ConsoleLogHandler()
        log_handler._console_view = self
        log.addHandler(log_handler)

    def isatty(self):
        """
        For exception handling purposes
        (see exception hook in the program entry point).
        """

        return False

    def onKeyPress_Tab(self):
        """
        Imitate cmd.Cmd.complete(self, text, state) function.
        """

        line = str(self.line).lstrip()
        cmd = line
        args = ''

        if len(self.line) > 0:
            cmd, args, _ = self.parseline(line)
            if cmd is None or cmd == '':
                compfunc = self.completedefault
            else:
                try:
                    compfunc = getattr(self, 'complete_' + cmd)
                except AttributeError:
                    compfunc = self.completenames
        else:
            compfunc = self.completenames

        self.completion_matches = compfunc(cmd, line, 0, 0)
        if self.completion_matches is not None:
            # Eliminate repeating values
            matches = []
            for m in self.completion_matches:
                try:
                    matches.index(m)
                except ValueError:
                    matches.append(m)

            # Update original list
            self.completion_matches = matches

            # In case we only have one possible value replace it on cmdline
            if len(self.completion_matches) == 1:
                newLine = self.completion_matches[0] + " " + args
                self.line = newLine
                self.point = len(newLine)
            # Else, display possible values
            elif self.completion_matches:
                self.write("\n")
                print(" ".join(self.completion_matches))

        # In any case, reprint prompt + line
        self.write("\n" + sys.ps1 + str(self.line))

    def writeNotification(self, message, details=""):
        """
        Write notification messages coming from the server.

        :param message: notification message
        :param details: details (text)
        """

        text = "Server notification: {}".format(message)
        if details:
            text += "\n" + details
        self.write_message_signal.emit(text, "info")

    def writeError(self, base_node_id, message):
        """
        Write error messages.

        :param base_node_id: base node identifier
        :param message: error message
        """

        node = Topology.instance().getNode(base_node_id)
        name = ""
        if node and node.name():
            name = " {}:".format(node.name())

        text = "Error:{name} {message}".format(name=name,
                                               message=message)
        self.write_message_signal.emit(text, "error")

    def writeWarning(self, base_node_id, message):
        """
        Write warning messages.

        :param base_node_id: base node identifier
        :param message: warning message
        """

        node = Topology.instance().getNode(base_node_id)
        name = ""
        if node and node.name():
            name = " {}:".format(node.name())

        text = "Warning:{name} {message}".format(name=name,
                                                 message=message)
        self.write_message_signal.emit(text, "warning")

    def writeServerError(self, base_node_id, message):
        """
        Write server error messages coming from the server.

        :param base_node_id: Base node identifier
        :param code: error code
        :param message: error message
        """

        node = Topology.instance().getNode(base_node_id)
        server = name = ""
        if node:
            if node.name():
                name = " {}:".format(node.name())
            server = "from {}".format(node.compute().name())

        text = "Server error {server}:{name} {message}".format(server=server,
                                                               name=name,
                                                               message=message)
        self.write_message_signal.emit(text.strip(), "error")

    def _run(self):
        """
        Runs as command as the cmd.Cmd class would do.
        PyCutExt was originally using as Interpreter to exec user's commands.
        Here we use directly the cmd.Cmd class.
        """

        self.pointer = 0
        if len(self.line):
            self.history.append(self.line)
            try:
                self.lines.append(self.line)
                source = "\n".join(self.lines)
                self.more = self.onecmd(source)
            except Exception as e:
                print("Unknown error: {}".format(e))

        self.write(self.prompt)
        self.lines = []
        self._clearLine()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import pytest
import unittest.mock
import pathlib
import tempfile

from gns3.registry.image import Image
from gns3.image_upload_manager import ImageUploadManager


@pytest.fixture
def image():
    (fd, path) = tempfile.mkstemp(suffix=".img")
    return Image('QEMU', path)


@pytest.fixture
def controller():
    return unittest.mock.MagicMock()

@pytest.fixture
def callback():
    return unittest.mock.MagicMock()

def test_upload_controller(image, controller, callback):
    manager = ImageUploadManager(image, controller, None)
    manager.upload()
    controller.post(
        f"/images/upload/{image.filename}",
        callback=callback,
        params={"image_type": image.type},
        body=pathlib.Path(image.path),
        context={"image_path": image.path},
        progress_text="Uploading {}".format(image.filename),
        timeout=None,
        wait=False
    )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Thread to copy or move files without blocking the GUI.
"""

import os
import shutil
from ..qt import QtCore

import logging
log = logging.getLogger(__name__)


class ProcessFilesWorker(QtCore.QObject):

    """
    Thread to process files (copy or move).

    :param source_dir: path to the source directory
    :param destination_dir: path to the destination directory (created if doesn't exist)
    :param move: indicates if the files must be moved instead of copied
    """

    # signals to update the progress dialog.
    error = QtCore.pyqtSignal(str, bool)
    finished = QtCore.pyqtSignal()
    updated = QtCore.pyqtSignal(int)

    def __init__(self, source_dir, destination_dir, move=False, skip_dirs=None, skip_files=None):

        super().__init__()
        self._is_running = False
        self._source = source_dir
        self._destination = destination_dir
        self._move = move
        self._skip_dirs = []
        self._skip_files = []
        if skip_dirs:
            self._skip_dirs = skip_dirs
        if skip_files:
            self._skip_files = skip_files

    def run(self):
        """
        Worker starting point.
        """

        self._is_running = True
        try:
            os.makedirs(self._destination)
        except FileExistsError:
            pass
        except OSError as e:
            self.error.emit("Could not create directory {}: {}".format(self._destination, e), True)
            return

        # Source can be None if directory have never been created (temporary project only on remote servers)
        if self._source is None:
            self.finished.emit()
            return

        copied = 0
        # start copying/moving from the source directory
        try:
            # count the number of files in the source directory
            file_count = self._countFiles(self._source)

            for path, dirs, filenames in os.walk(self._source):
                dirs[:] = [d for d in dirs if d not in self._skip_dirs]
                filenames[:] = [f for f in filenames if f not in self._skip_files]
                base_dir = path.replace(self._source, self._destination)

                # start create the destination sub-directories
                for directory in dirs:
                    try:
                        destination_dir = os.path.join(base_dir, directory)
                        os.makedirs(destination_dir)
                    except FileExistsError:
                        pass
                    except OSError as e:
                        self.error.emit("Could not create directory {}: {}".format(destination_dir, e), True)
                        return
                    if not self._is_running:
                        return

                # finally the files themselves
                for sfile in filenames:
                    source_file = os.path.join(path, sfile)
                    destination_file = os.path.join(base_dir, sfile)
                    try:
                        if self._move:
                            shutil.move(source_file, destination_file)
                        else:
                            shutil.copy2(source_file, destination_file)
                    except OSError as e:
                        if self._move:
                            log.warning("Cannot move: {}".format(e))
                            self.error.emit("Could not move file to {}: {}".format(destination_file, e), False)
                        else:
                            log.warning("Cannot copy: {}".format(e))
                            self.error.emit("Could not copy file to {}: {}".format(destination_file, e), False)
                    if not self._is_running:
                        return
                    copied += 1
                    # update the progress made
                    progress = float(copied) / file_count * 100
                    self.updated.emit(progress)
        except RuntimeError:
            self.error.emit("Maximum path depth exceedeed when copying {}".format(self._source), True)

        # everything has been copied or moved, let's inform the GUI
        self.finished.emit()

    def _countFiles(self, directory):
        """
        Counts all the files in a directory.

        :param directory: path to the directory.
        """

        count = 0
        for _, dirs, files in os.walk(directory):
            dirs[:] = [d for d in dirs if d not in self._skip_dirs]
            files[:] = [f for f in files if f not in self._skip_files]
            count += len(files)
        return count

    def cancel(self):
        """
        Cancel this worker.
        """

        if not self:
            return
        self._is_running = False



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import shutil

from gns3.qt import QtWidgets
from gns3.local_config import LocalConfig
from gns3.ui.profile_select_dialog_ui import Ui_ProfileSelectDialog
from gns3.version import __version_info__

import logging
log = logging.getLogger(__name__)


class ProfileSelectDialog(QtWidgets.QDialog, Ui_ProfileSelectDialog):
    """
    This dialog allow user to choose a profile of settings
    """

    def __init__(self, parent=None):

        if parent is None:
            self._main = QtWidgets.QMainWindow()
            self._main.hide()
            parent = self._main
        super().__init__(parent)

        self.setupUi(self)
        self.uiNewPushButton.clicked.connect(self._newPushButtonSlot)
        self.uiDeletePushButton.clicked.connect(self._deletePushButtonSlot)

        # Center on screen
        screen = QtWidgets.QApplication.desktop().screenGeometry()
        self.move(screen.center() - self.rect().center())

        version = "{}.{}".format(__version_info__[0], __version_info__[1])
        if sys.platform.startswith("win"):
            appdata = os.path.expandvars("%APPDATA%")
            path = os.path.join(appdata, "GNS3", version)
        else:
            home = os.path.expanduser("~")
            path = os.path.join(home, ".config", "GNS3", version)
        self.profiles_path = os.path.join(path, "profiles")

        self.uiShowAtStartupCheckBox.setChecked(LocalConfig.instance().multiProfiles())
        self._refresh()

    def _refresh(self):
        self.uiProfileSelectComboBox.clear()
        self.uiProfileSelectComboBox.addItem("default")

        try:
            if os.path.exists(self.profiles_path):
                for profile in sorted(os.listdir(self.profiles_path)):
                    if not profile.startswith("."):
                        self.uiProfileSelectComboBox.addItem(profile)
        except OSError:
            pass

    def profile(self):
        return self.uiProfileSelectComboBox.currentText()

    def accept(self):
        LocalConfig.instance().setMultiProfiles(self.uiShowAtStartupCheckBox.isChecked())
        super().accept()

    def _newPushButtonSlot(self):
        profile, ok = QtWidgets.QInputDialog.getText(self, "New profile", "Profile name:")
        if ok:
            self.uiProfileSelectComboBox.addItem(profile)
            self.uiProfileSelectComboBox.setCurrentText(profile)
            self.accept()

    def _deletePushButtonSlot(self):
        profile = self.uiProfileSelectComboBox.currentText()
        if profile == "default":
            QtWidgets.QMessageBox.critical(self, "Delete profile", "The default profile cannot be deleted")
        else:
            try:
                shutil.rmtree(os.path.join(self.profiles_path, profile))
                self._refresh()
            except (OSError, PermissionError) as e:
                QtWidgets.QMessageBox.critical(self, "Cannot delete profile", str(e))


if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    dialog = ProfileSelectDialog()
    dialog.show()
    exit_code = app.exec_()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/builtin/ui/ethernet_hub_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_EthernetHubPreferencesPageWidget(object):
    def setupUi(self, EthernetHubPreferencesPageWidget):
        EthernetHubPreferencesPageWidget.setObjectName("EthernetHubPreferencesPageWidget")
        EthernetHubPreferencesPageWidget.resize(546, 455)
        EthernetHubPreferencesPageWidget.setAccessibleDescription("")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(EthernetHubPreferencesPageWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.splitter = QtWidgets.QSplitter(EthernetHubPreferencesPageWidget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.uiEthernetHubsTreeWidget = QtWidgets.QTreeWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiEthernetHubsTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiEthernetHubsTreeWidget.setSizePolicy(sizePolicy)
        self.uiEthernetHubsTreeWidget.setMaximumSize(QtCore.QSize(160, 16777215))
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.uiEthernetHubsTreeWidget.setFont(font)
        self.uiEthernetHubsTreeWidget.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.uiEthernetHubsTreeWidget.setIconSize(QtCore.QSize(32, 32))
        self.uiEthernetHubsTreeWidget.setRootIsDecorated(False)
        self.uiEthernetHubsTreeWidget.setObjectName("uiEthernetHubsTreeWidget")
        self.uiEthernetHubsTreeWidget.headerItem().setText(0, "1")
        self.uiEthernetHubsTreeWidget.header().setVisible(False)
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiEthernetHubInfoTreeWidget = QtWidgets.QTreeWidget(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiEthernetHubInfoTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiEthernetHubInfoTreeWidget.setSizePolicy(sizePolicy)
        self.uiEthernetHubInfoTreeWidget.setIndentation(10)
        self.uiEthernetHubInfoTreeWidget.setAllColumnsShowFocus(True)
        self.uiEthernetHubInfoTreeWidget.setObjectName("uiEthernetHubInfoTreeWidget")
        self.uiEthernetHubInfoTreeWidget.header().setVisible(False)
        self.verticalLayout.addWidget(self.uiEthernetHubInfoTreeWidget)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiNewEthernetHubPushButton = QtWidgets.QPushButton(self.layoutWidget)
        self.uiNewEthernetHubPushButton.setObjectName("uiNewEthernetHubPushButton")
        self.horizontalLayout_5.addWidget(self.uiNewEthernetHubPushButton)
        self.uiEditEthernetHubPushButton = QtWidgets.QPushButton(self.layoutWidget)
        self.uiEditEthernetHubPushButton.setEnabled(False)
        self.uiEditEthernetHubPushButton.setObjectName("uiEditEthernetHubPushButton")
        self.horizontalLayout_5.addWidget(self.uiEditEthernetHubPushButton)
        self.uiDeleteEthernetHubPushButton = QtWidgets.QPushButton(self.layoutWidget)
        self.uiDeleteEthernetHubPushButton.setEnabled(False)
        self.uiDeleteEthernetHubPushButton.setObjectName("uiDeleteEthernetHubPushButton")
        self.horizontalLayout_5.addWidget(self.uiDeleteEthernetHubPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.verticalLayout_2.addWidget(self.splitter)

        self.retranslateUi(EthernetHubPreferencesPageWidget)
        QtCore.QMetaObject.connectSlotsByName(EthernetHubPreferencesPageWidget)
        EthernetHubPreferencesPageWidget.setTabOrder(self.uiNewEthernetHubPushButton, self.uiDeleteEthernetHubPushButton)

    def retranslateUi(self, EthernetHubPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        EthernetHubPreferencesPageWidget.setWindowTitle(_translate("EthernetHubPreferencesPageWidget", "Ethernet hubs"))
        EthernetHubPreferencesPageWidget.setAccessibleName(_translate("EthernetHubPreferencesPageWidget", "Ethernet hub templates"))
        self.uiEthernetHubInfoTreeWidget.headerItem().setText(0, _translate("EthernetHubPreferencesPageWidget", "1"))
        self.uiEthernetHubInfoTreeWidget.headerItem().setText(1, _translate("EthernetHubPreferencesPageWidget", "2"))
        self.uiNewEthernetHubPushButton.setText(_translate("EthernetHubPreferencesPageWidget", "&New"))
        self.uiEditEthernetHubPushButton.setText(_translate("EthernetHubPreferencesPageWidget", "&Edit"))
        self.uiDeleteEthernetHubPushButton.setText(_translate("EthernetHubPreferencesPageWidget", "&Delete"))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Default Built-in settings.
"""

import sys

from gns3.node import Node

if sys.platform.startswith("linux"):
    DEFAULT_NAT_INTERFACE = "virbr0"
else:
    DEFAULT_NAT_INTERFACE = "vmnet8"

BUILTIN_SETTINGS = {
    "default_nat_interface": DEFAULT_NAT_INTERFACE
}

CLOUD_SETTINGS = {
    "name": "",
    "usage": "",
    "remote_console_host": "127.0.0.1",
    "remote_console_port": 23,
    "remote_console_type": "none",
    "remote_console_http_path": "/",
    "default_name_format": "Cloud{0}",
    "symbol": "cloud",
    "category": Node.end_devices,
    "ports_mapping": [],
    "node_type": "cloud"
}

ETHERNET_HUB_SETTINGS = {
    "name": "",
    "default_name_format": "Hub{0}",
    "symbol": "hub",
    "category": Node.switches,
    "ports_mapping": [],
    "node_type": "ethernet_hub"
}

ETHERNET_SWITCH_SETTINGS = {
    "name": "",
    "default_name_format": "Switch{0}",
    "symbol": "ethernet_switch",
    "category": Node.switches,
    "console_type": "none",
    "ports_mapping": [],
    "node_type": "ethernet_switch"
}



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Display error to the user in an overlay popup
"""

import time

from gns3.qt import QtWidgets, QtCore, qslot

import logging
log = logging.getLogger(__name__)


MAX_ELEMENTS = 3
DISPLAY_DURATION = {
    "CRITICAL": 120,
    "ERROR": 120,
    "WARNING": 20,
    "INFO": 5
}


class NotifDialogHandler(logging.StreamHandler):

    def __init__(self, dialog):
        super().__init__()
        self._dialog = dialog
        self.setLevel(logging.INFO)
        self._dialog.show()

    def emit(self, record):
        self._dialog.addNotif(record.levelname, record.getMessage())


class NotifDialog(QtWidgets.QWidget):

    def __init__(self, parent):
        super().__init__(parent)
        self._notifs = []

        self.setWindowFlags(QtCore.Qt.FramelessWindowHint |
                            QtCore.Qt.WindowDoesNotAcceptFocus |
                            QtCore.Qt.SubWindow)
        # QtCore.Qt.Tool)
        # QtCore.Qt.WindowStaysOnTopHint)
        self.setAttribute(QtCore.Qt.WA_ShowWithoutActivating)  # | QtCore.Qt.WA_TranslucentBackground)

        self._layout = QtWidgets.QVBoxLayout()

        self._timer = QtCore.QTimer()
        self._timer.setInterval(1000)
        self._timer.timeout.connect(self._refreshSlot)
        self._timer.start()

        for i in range(0, MAX_ELEMENTS):
            l = QtWidgets.QLabel()
            l.setAlignment(QtCore.Qt.AlignTop)
            l.setWordWrap(True)
            l.hide()
            self._layout.addWidget(l)
        self.setLayout(self._layout)

    @qslot
    def addNotif(self, level, message):
        if not self.parent().settings().get("overlay_notifications", True):
            return

        # This unicode char prevent the wordwrap at /
        message = message.replace("/", "\u2060/\u2060")
        if len(self._notifs) == MAX_ELEMENTS:
            self._notifs.pop(0)
        self._notifs.append((level, message, time.time()))
        self.update()

    @qslot
    def _refreshSlot(self):
        """
        Hide the notifs after some delay
        """
        notifs = []
        for (i, (level, message, when)) in enumerate(self._notifs):
            if when + DISPLAY_DURATION[level] > time.time():
                notifs.append((level, message, when))
        if notifs != self._notifs:
            self._notifs = notifs
            self.update()
        elif len(notifs) > 0:
            self.resize()

    def update(self):
        if len(self._notifs) == 0:
            self.hide()
        else:
            for (i, (level, message, when)) in enumerate(self._notifs):
                w = self._layout.itemAt(i).widget()
                w.setText(message)
                if level == "ERROR" or level == "CRITICAL":
                    w.setStyleSheet("""
                        color: black;
                        padding-left: 12px;
                        background-color: rgb(247, 205, 198);
                        border-left: 10px solid red;
                    """)
                elif level == "WARNING":
                    w.setStyleSheet("""
                        color: black;
                        padding-left: 12px;
                        background-color: #f4f2b5;
                        border-left: 10px solid orange;
                    """)
                elif level == "INFO":
                    w.setStyleSheet("""
                        color: black;
                        padding-left: 12px;
                        background-color: #cfffc9;
                        border-left: 10px solid green;
                    """)

                w.show()
            for i in range(i + 1, MAX_ELEMENTS):
                w = self._layout.itemAt(i).widget()
                w.hide()

            self.resize()
            self.show()

    def resize(self):
        x = self.parent().width() - self.width() - 10
        y = 10
        self.setGeometry(x, y, self.sizeHint().width(), self.sizeHint().height())

    @qslot
    def mousePressEvent(self, event):
        self._notifs.clear()
        self.update()


if __name__ == '__main__':
    """
    A demo main for testing the features
    """
    import sys
    app = QtWidgets.QApplication(sys.argv)
    logging.basicConfig(level=logging.INFO)

    class MainWindow(QtWidgets.QWidget):

        def __init__(self):
            super().__init__()
            l1 = QtWidgets.QLabel()
            l1.setText("Hello World")

            vbox = QtWidgets.QVBoxLayout()
            vbox.addWidget(l1)
            self.setLayout(vbox)
            self.setStyleSheet("background-color:blue;")
            self._dialog = NotifDialog(self)
            log.addHandler(NotifDialogHandler(self._dialog))
            log.info("test")

        def moveEvent(self, event):
            log.error("An error")
            log.info("An info with an url http://test")
            log.warning("A warning with a long long long longlong longlong longlong longlong longlong longlong longlong long message")
            self._dialog.update()

        def resizeEvent(self, event):
            self._dialog.update()

    main = MainWindow()
    main.setMinimumWidth(600)
    main.setMinimumHeight(600)
    main.show()
    exit_code = app.exec_()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import pathlib

from gns3.http_client_error import HttpClientError, HttpClientCancelledRequestError
from gns3.qt import QtWidgets
from gns3.registry.image import Image
from gns3.controller import Controller

import logging
log = logging.getLogger(__name__)


class ImageUploadManager(object):
    """
    Manager over the image upload
    """

    def __init__(self, image: Image, controller: Controller, parent: QtWidgets.QWidget):

        self._image = image
        self._controller = controller
        self._parent = parent

    def upload(self) -> bool:

        if not os.path.exists(self._image.path):
            log.error("Image '{}' could not be found".format(self._image.path))
            return False
        return self._fileUploadToController()

    def _fileUploadToController(self) -> bool:

        log.debug("Uploading image '{}' to controller".format(self._image.path))
        try:
            self._controller.post(
                f"/images/upload/{self._image.filename}",
                callback=None,
                body=pathlib.Path(self._image.path),
                context={"image_path": self._image.path},
                progress_text="Uploading {}".format(self._image.filename),
                timeout=None,
                wait=True
            )
        except HttpClientCancelledRequestError:
            return False
        except HttpClientError as e:
            QtWidgets.QMessageBox.critical(
                self._parent,
                "Image upload",
                f"Could not upload image {self._image.filename}: {e}"
            )
            return False
        return True



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Wizard for QEMU disk images.
"""

from gns3.qt import QtCore, QtGui, QtWidgets
from ..ui.qemu_image_wizard_ui import Ui_QemuImageWizard


class QemuImageWizard(QtWidgets.QWizard, Ui_QemuImageWizard):

    """
    Wizard to create a Qemu VM.

    :param parent: parent widget
    :param node_or_server: node or server where the image should be created
    :param filename: Default filename of image.
    :param size: Default size (in MiB) for the image.
    """

    def __init__(self, parent, node_or_server, filename="disk", size=30000):

        super().__init__(parent)

        self._node_or_server = node_or_server
        self.setupUi(self)
        self.setPixmap(QtWidgets.QWizard.LogoPixmap, QtGui.QPixmap(":/symbols/qemu_guest.svg"))

        # Initialize "constants"
        self._mappings = {
            self.uiFormatQcow2Radio: ("qcow2", ".qcow2", self.uiQcow2OptionsWizardPage),
            self.uiFormatQcowRadio: ("qcow", ".qcow", None),
            self.uiFormatVhdRadio: ("vpc", ".vhd", self.uiVhdOptionsWizardPage),
            self.uiFormatVdiRadio: ("vdi", ".vdi", self.uiVdiOptionsWizardPage),
            self.uiFormatVmdkRadio: ("vmdk", ".vmdk", self.uiVmdkOptionsWizardPage),
            self.uiFormatRawRadio: ("raw", ".img", None)
        }

        # isComplete() overrides
        self.uiNameAndSizeWizardPage.isComplete = self._uiNameAndSizeWizardPage_isComplete
        self.uiFormatWizardPage.isComplete = self._uiFormatWizardPage_isComplete

        # Signal connections
        self.uiFormatRadios.buttonClicked.connect(self._formatChangedSlot)
        self.uiDiskFilenameLineEdit.textChanged.connect(self._filenameChangedSlot)

        # Finish setup
        self.page(self.pageIds()[-1]).validatePage = self._createDisk
        self.uiDiskFilenameLineEdit.setText(filename)
        self.uiSizeSpinBox.setValue(size)
        self._formatChangedSlot(self.uiFormatQcow2Radio)

    def _getCreateDiskServerCallback(self, result, error=False, **kwargs):
        """
        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            QtWidgets.QMessageBox.critical(self, "Create disk", "{}".format(result["message"]))

    def _uiNameAndSizeWizardPage_isComplete(self):

        return not "" == self.uiDiskFilenameLineEdit.text()

    def _uiFormatWizardPage_isComplete(self):

        return self.uiFormatRadios.checkedButton() is not None

    def _filenameChangedSlot(self, new_value):

        self.uiNameAndSizeWizardPage.completeChanged.emit()

    def _formatChangedSlot(self, new_format):

        filename = self.uiDiskFilenameLineEdit.text().strip()
        try:
            filename = filename[:filename.rindex('.')] + self._mappings[new_format][1]
        except ValueError:
            # The file has no extension; Just give it one
            filename = filename + self._mappings[new_format][1]
        self.uiDiskFilenameLineEdit.setText(filename)
        self.uiFormatWizardPage.completeChanged.emit()

    def _createDisk(self):

        selected_format = self.uiFormatRadios.checkedButton()
        disk_image_filename = self.uiDiskFilenameLineEdit.text().strip()
        options = {}
        options["format"] = self._mappings[selected_format][0]
        options["size"] = self.uiSizeSpinBox.value()

        if selected_format == self.uiFormatQcow2Radio:
            preallocation = self.uiQcow2PreallocationRadios.checkedButton()
            if preallocation is not None:
                preallocation_mappings = {
                    self.uiQcow2PreallocationOffRadio: 'off',
                    self.uiQcow2PreallocationMetadataRadio: 'metadata',
                    self.uiQcow2PreallocationFallocRadio: 'falloc',
                    self.uiQcow2PreallocationFullRadio: 'full'
                }
                options["preallocation"] = preallocation_mappings[preallocation]

            cluster_size = self.uiQcow2ClusterSizeComboBox.currentText()
            if not '<default>' == cluster_size:
                if cluster_size.endswith('k'):
                    options["cluster_size"] = int(cluster_size[:-1]) * 1024
                else:
                    options["cluster_size"] = int(cluster_size)

            refcount_bits = self.uiRefcountEntrySizeComboBox.currentText()
            if not '<default>' == refcount_bits:
                options["refcount_bits"] = int(refcount_bits)

            options["lazy_refcounts"] = 'on' if QtCore.Qt.Checked == self.uiLazyRefcountsCheckBox.checkState() else 'off'

        elif selected_format == self.uiFormatVhdRadio:
            size_mode = self.uiVhdSizeModeRadios.checkedButton()
            if size_mode is not None:
                size_mode_mappings = {
                    self.uiVhdFileSizeModeDynamicRadio: 'dynamic',
                    self.uiVhdFileSizeModeFixedRadio: 'fixed'
                }
                options['subformat'] = size_mode_mappings[size_mode]

        elif selected_format == self.uiFormatVdiRadio:
            size_mode = self.uiVhdSizeModeRadios.checkedButton()
            if size_mode is not None:
                options["static"] = 'on' if size_mode == self.uiVhdFileSizeModeFixedRadio else 'off'

        elif selected_format == self.uiFormatVmdkRadio:
            options["zeroed_grain"] = 'on' if QtCore.Qt.Checked == self.uiVmdkZeroedGrainCheckBox.checkState() else 'off'

            adapter_type = self.uiVmdkAdapterRadios.checkedButton()
            if adapter_type is not None:
                adapter_type_mappings = {
                    self.uiVmdkAdapterTypeIdeRadio: 'ide',
                    self.uiVmdkAdapterTypeLsiRadio: 'lsilogic',
                    self.uiVmdkAdapterTypeBusRadio: 'buslogic',
                    self.uiVmdkAdapterTypeEsxRadio: 'legacyESX'
                }
                options['adapter_type'] = adapter_type_mappings[adapter_type]

            stream_optimized = self.uiVmdkStreamOptimizedCheckBox.checkState()
            if QtCore.Qt.Checked == stream_optimized:
                options['subformat'] = 'streamOptimized'
            else:
                two = 'twoGbMaxExtent' if QtCore.Qt.Checked == self.uiVmdkSplit2gCheckBox.checkState() else 'monolithic'
                size_mode = self.uiVmdkSizeModeRadios.checkedButton()
                if size_mode is not None or two == 'twoGbMaxExtent':
                    if size_mode is None:
                        size_mode = self.uiVmdkFileSizeModeSparseRadio

                    size_mode_mappings = {
                        self.uiVmdkFileSizeModeSparseRadio: 'Sparse',
                        self.uiVmdkFileSizeModeFlatRadio: 'Flat'
                    }
                    options['subformat'] = two + size_mode_mappings[size_mode]

        self._node_or_server.createDiskImage(disk_image_filename, options, self._getCreateDiskServerCallback)
        return True

    def nextId(self):

        if self.page(self.currentId()) == self.uiFormatWizardPage:
            current_format = self.uiFormatRadios.checkedButton()
            if not current_format:
                return self.currentId()
            next_page = self._mappings[current_format][2]
            if next_page:
                return next_page.nextId() - 1
        default_nextId = super().nextId()
        if not default_nextId == -1:
            return self.pageIds()[-1]
        return default_nextId



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import shutil
from ssl import CertificateError

from ..qt import QtCore, QtWidgets, QtNetwork
from ..controller import Controller
from .config import Config, ConfigException


import logging
log = logging.getLogger(__name__)


class ApplianceToTemplate:
    """
    Appliance installation.
    """

    def new_template(self, appliance_config, server, appliance_version=None, controller_symbols=None, parent=None):
        """
        Creates a new template from an appliance.

        :param appliance_config: Dictionary with appliance configuration
        :param server
        """

        self._parent = parent
        self._registry_version = appliance_config["registry_version"]
        new_template = {
            "compute_id": server,
            "name": appliance_config["name"]
        }

        if "usage" in appliance_config:
            new_template["usage"] = appliance_config["usage"]

        if appliance_config["category"] == "multilayer_switch":
            new_template["category"] = "switch"
        else:
            new_template["category"] = appliance_config["category"]

        if "symbol" in appliance_config:
            new_template["symbol"] = self._set_symbol(appliance_config["symbol"], controller_symbols)

        if new_template.get("symbol") is None:
            if appliance_config["category"] == "guest":
                if "docker" in appliance_config:
                    new_template["symbol"] = "docker_guest"
                else:
                    new_template["symbol"] = "qemu_guest"
            elif appliance_config["category"] == "switch":
                new_template["symbol"] = "ethernet_switch"
            else:
                new_template["symbol"] = appliance_config["category"]

        if self._registry_version >= 8:
            if appliance_version:
                for version in appliance_config["versions"]:
                    if appliance_version and version["name"] == appliance_version:
                        # inject "usage", "category" and "symbol" specified at the version
                        # level into the template properties
                        usage = version.get("usage")
                        if usage:
                            new_template["usage"] = usage
                        new_template["symbol"] = version.get("symbol", new_template["symbol"])
                        new_template["category"] = version.get("category", new_template["category"])
                        settings = self._get_settings(appliance_config, version.get("settings"))
                        template_type = settings["template_type"]
                        if template_type == "qemu":
                            self._add_qemu_config(new_template, settings["template_properties"], appliance_config)
                        elif template_type == "iou":
                            self._add_iou_config(new_template, settings["template_properties"], appliance_config)
                        elif template_type == "dynamips":
                            self._add_dynamips_config(new_template, settings["template_properties"], appliance_config)
            else:
                # docker appliances have no version
                settings = self._get_settings(appliance_config)
                if settings["template_type"] == "docker":
                    self._add_docker_config(new_template, settings["template_properties"], appliance_config)
        else:
            if "qemu" in appliance_config:
                self._add_qemu_config(new_template, appliance_config["qemu"], appliance_config)
            elif "iou" in appliance_config:
                self._add_iou_config(new_template, appliance_config["iou"], appliance_config)
            elif "dynamips" in appliance_config:
                self._add_dynamips_config(new_template, appliance_config["dynamips"], appliance_config)
            elif "docker" in appliance_config:
                self._add_docker_config(new_template, appliance_config["docker"], appliance_config)
            else:
                raise ConfigException("{} no configuration found for known emulators".format(new_template["name"]))

        return new_template

    def _get_settings(self, appliance_config, settings_name=None):

        default_settings = None
        # first look for default settings, if any ('default' = true, first set that has it)
        for settings in appliance_config["settings"]:
            if settings.get("default", False):
                default_settings = settings
                break

        # then look for specific settings set if a name is provided
        if settings_name:
            for settings in appliance_config["settings"]:
                if settings.get("name") == settings_name:
                    if settings.get("inherit_default_properties", True) and \
                            default_settings and default_settings["template_type"] == settings["template_type"]:
                        default_settings["template_properties"].update(settings["template_properties"])
                        return default_settings
                    return settings
            raise ConfigException("Settings '{}' cannot be found in the appliance file", settings_name)
        elif default_settings:
            return default_settings

        if not appliance_config.get("settings"):
            raise ConfigException("No settings found in the appliance file")

        # if no default settings are specified, use the first available settings set
        return appliance_config["settings"][0]

    def _add_qemu_config(self, new_config, template_properties, appliance_config):

        new_config["template_type"] = "qemu"
        new_config.update(template_properties)

        # the following properties are not valid for a template
        new_config.pop("kvm", None)  # To check KVM setting against the server capabilities
        new_config.pop("path", None)  # Qemu binary selected in previous step
        new_config.pop("arch", None)  # Used for selecting the Qemu binary

        options = template_properties.get("options", "")
        if template_properties.get("kvm", "allow") == "disable" and "-machine accel=tcg" not in options:
            options += " -machine accel=tcg"
        options = options.strip()
        if options:
            new_config["options"] = options

        for image in appliance_config["images"]:
            if image.get("path"):
                new_config[image["type"]] = self._relative_image_path("QEMU", image["path"])

        if "arch" in appliance_config:
            new_config["platform"] = appliance_config["arch"]

        if "first_port_name" in appliance_config:
            new_config["first_port_name"] = appliance_config["first_port_name"]

        if "port_name_format" in appliance_config:
            new_config["port_name_format"] = appliance_config["port_name_format"]

        if "port_segment_size" in appliance_config:
            new_config["port_segment_size"] = appliance_config["port_segment_size"]

        if "custom_adapters" in appliance_config:
            new_config["custom_adapters"] = appliance_config["custom_adapters"]

        if "linked_clone" in appliance_config:
            new_config["linked_clone"] = appliance_config["linked_clone"]

    def _add_docker_config(self, new_config, template_properties, appliance_config):

        new_config["template_type"] = "docker"
        new_config.update(template_properties)

        if "custom_adapters" in appliance_config:
            new_config["custom_adapters"] = appliance_config["custom_adapters"]

    def _add_dynamips_config(self, new_config, template_properties, appliance_config):

        new_config["template_type"] = "dynamips"
        new_config.update(template_properties)

        for image in appliance_config["images"]:
            new_config[image["type"]] = self._relative_image_path("IOS", image["path"])
            if self._registry_version < 8:
                new_config["idlepc"] = image.get("idlepc", "")

    def _add_iou_config(self, new_config, template_properties, appliance_config):

        new_config["template_type"] = "iou"
        new_config.update(template_properties)
        for image in appliance_config["images"]:
            if "path" not in image:
                raise ConfigException("Disk image is missing")
            new_config[image["type"]] = self._relative_image_path("IOU", image["path"])
        new_config["path"] = new_config["image"]

    def _relative_image_path(self, image_dir_type, path):
        """

        :param image_dir_type: Type of image directory
        :param filename: Filename at the end of the process
        :param path: Full path to the file
        :returns: Path relative to image directory.
        Copy the image to the directory if not already in the directory
        """

        images_dir = os.path.join(Config().images_dir, image_dir_type)
        path = os.path.abspath(path)
        if os.path.commonprefix([images_dir, path]) == images_dir:
            return path.replace(images_dir, '').strip('/\\')

        return os.path.basename(path)

    def _set_symbol(self, symbol_id, controller_symbols):
        """
        Check if exists on controller or download symbol from the web if needed
        """

        # GNS3 builtin symbol
        if symbol_id.startswith(":/symbols/"):
            return symbol_id

        controller = Controller.instance()
        path = os.path.join(Config().symbols_dir, symbol_id)
        if not controller.isRemote() and os.path.exists(path):
            return os.path.basename(path)

        if controller_symbols:
            is_symbol_on_controller = len([s for s in controller_symbols if s['symbol_id'] == symbol_id]) > 0

            if is_symbol_on_controller:
                cached = Controller.instance().getStaticCachedPath(symbol_id)
                if os.path.exists(cached):
                    try:
                        shutil.copy(cached, path)
                    except IOError as e:
                        log.warning("Cannot copy cached symbol from `{}` to `{}` due `{}`".format(cached, path, e))
                return symbol_id

        url = "https://raw.githubusercontent.com/GNS3/gns3-registry/master/symbols/{}".format(symbol_id)
        try:
            if not self._downloadApplianceSymbol(url, path):
                return None
            controller.clearStaticCache()
            if controller.isRemote():
                controller.uploadSymbol(symbol_id, path)
            return os.path.basename(path)
        except (OSError, CertificateError):
            return None

    def _downloadApplianceSymbol(self, url, path, timeout=30):
        """
        Download an appliance symbol in a synchronous way.
        """

        network_manager = QtNetwork.QNetworkAccessManager()
        request = QtNetwork.QNetworkRequest(QtCore.QUrl(url))
        request.setRawHeader(b'User-Agent', b'GNS3 symbol downloader')
        reply = network_manager.get(request)
        progress_dialog = QtWidgets.QProgressDialog("Downloading '{}' appliance symbol...".format(os.path.basename(path)), "Cancel", 0, 0, self._parent)
        progress_dialog.setMinimumDuration(0)
        reply.finished.connect(progress_dialog.close)
        QtCore.QTimer.singleShot(timeout * 1000, progress_dialog.close)
        log.debug("Downloading appliance symbol from '{}'".format(url))
        progress_dialog.show()
        progress_dialog.exec_()
        status = reply.attribute(QtNetwork.QNetworkRequest.HttpStatusCodeAttribute)
        if reply.error() == QtNetwork.QNetworkReply.NoError and status == 200:
            try:
                with open(path, 'wb+') as f:
                    f.write(reply.readAll())
            except OSError as e:
                log.debug("Error while saving appliance symbol to '{}': {}".format(path, e))
                raise
            log.debug("Appliance symbol downloaded and saved to '{}'".format(path))
            return True
        else:
            log.error("Error when downloading appliance symbol from '{}': {}".format(url, reply.errorString()))
        return False



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/virtualbox/ui/virtualbox_vm_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_VirtualBoxVMPreferencesPageWidget(object):
    def setupUi(self, VirtualBoxVMPreferencesPageWidget):
        VirtualBoxVMPreferencesPageWidget.setObjectName("VirtualBoxVMPreferencesPageWidget")
        VirtualBoxVMPreferencesPageWidget.resize(572, 486)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(VirtualBoxVMPreferencesPageWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.splitter = QtWidgets.QSplitter(VirtualBoxVMPreferencesPageWidget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.uiVirtualBoxVMsTreeWidget = QtWidgets.QTreeWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVirtualBoxVMsTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiVirtualBoxVMsTreeWidget.setSizePolicy(sizePolicy)
        self.uiVirtualBoxVMsTreeWidget.setMaximumSize(QtCore.QSize(160, 16777215))
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.uiVirtualBoxVMsTreeWidget.setFont(font)
        self.uiVirtualBoxVMsTreeWidget.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.uiVirtualBoxVMsTreeWidget.setIconSize(QtCore.QSize(32, 32))
        self.uiVirtualBoxVMsTreeWidget.setRootIsDecorated(False)
        self.uiVirtualBoxVMsTreeWidget.setObjectName("uiVirtualBoxVMsTreeWidget")
        self.uiVirtualBoxVMsTreeWidget.headerItem().setText(0, "1")
        self.uiVirtualBoxVMsTreeWidget.header().setVisible(False)
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiVirtualBoxVMInfoTreeWidget = QtWidgets.QTreeWidget(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVirtualBoxVMInfoTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiVirtualBoxVMInfoTreeWidget.setSizePolicy(sizePolicy)
        self.uiVirtualBoxVMInfoTreeWidget.setIndentation(10)
        self.uiVirtualBoxVMInfoTreeWidget.setAllColumnsShowFocus(True)
        self.uiVirtualBoxVMInfoTreeWidget.setObjectName("uiVirtualBoxVMInfoTreeWidget")
        self.uiVirtualBoxVMInfoTreeWidget.header().setVisible(False)
        self.verticalLayout.addWidget(self.uiVirtualBoxVMInfoTreeWidget)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiNewVirtualBoxVMPushButton = QtWidgets.QPushButton(self.layoutWidget)
        self.uiNewVirtualBoxVMPushButton.setObjectName("uiNewVirtualBoxVMPushButton")
        self.horizontalLayout_5.addWidget(self.uiNewVirtualBoxVMPushButton)
        self.uiEditVirtualBoxVMPushButton = QtWidgets.QPushButton(self.layoutWidget)
        self.uiEditVirtualBoxVMPushButton.setEnabled(False)
        self.uiEditVirtualBoxVMPushButton.setObjectName("uiEditVirtualBoxVMPushButton")
        self.horizontalLayout_5.addWidget(self.uiEditVirtualBoxVMPushButton)
        self.uiDeleteVirtualBoxVMPushButton = QtWidgets.QPushButton(self.layoutWidget)
        self.uiDeleteVirtualBoxVMPushButton.setEnabled(False)
        self.uiDeleteVirtualBoxVMPushButton.setObjectName("uiDeleteVirtualBoxVMPushButton")
        self.horizontalLayout_5.addWidget(self.uiDeleteVirtualBoxVMPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.verticalLayout_2.addWidget(self.splitter)

        self.retranslateUi(VirtualBoxVMPreferencesPageWidget)
        QtCore.QMetaObject.connectSlotsByName(VirtualBoxVMPreferencesPageWidget)
        VirtualBoxVMPreferencesPageWidget.setTabOrder(self.uiNewVirtualBoxVMPushButton, self.uiDeleteVirtualBoxVMPushButton)

    def retranslateUi(self, VirtualBoxVMPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        VirtualBoxVMPreferencesPageWidget.setWindowTitle(_translate("VirtualBoxVMPreferencesPageWidget", "VirtualBox VMs"))
        VirtualBoxVMPreferencesPageWidget.setAccessibleName(_translate("VirtualBoxVMPreferencesPageWidget", "VirtualBox VM templates"))
        self.uiVirtualBoxVMInfoTreeWidget.headerItem().setText(0, _translate("VirtualBoxVMPreferencesPageWidget", "1"))
        self.uiVirtualBoxVMInfoTreeWidget.headerItem().setText(1, _translate("VirtualBoxVMPreferencesPageWidget", "2"))
        self.uiNewVirtualBoxVMPushButton.setText(_translate("VirtualBoxVMPreferencesPageWidget", "&New"))
        self.uiEditVirtualBoxVMPushButton.setText(_translate("VirtualBoxVMPreferencesPageWidget", "&Edit"))
        self.uiDeleteVirtualBoxVMPushButton.setText(_translate("VirtualBoxVMPreferencesPageWidget", "&Delete"))




#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .qt import QtCore, QtWidgets
from .compute import Compute
from .controller import Controller

import sys
import copy
import urllib
import datetime


import logging
log = logging.getLogger(__name__)


class ComputeManager(QtCore.QObject):
    """
    Manager for computes.
    """

    created_signal = QtCore.Signal(str)
    updated_signal = QtCore.Signal(str)
    deleted_signal = QtCore.Signal(str)

    def __init__(self):

        super().__init__()
        self._computes = {}
        self._controller = Controller.instance()
        self._controller.connected_signal.connect(self._controllerConnectedSlot)
        self._controller.disconnected_signal.connect(self._controllerDisconnectedSlot)
        self._controllerConnectedSlot()

        # No need to refresh via an API call if we received fresh data from the notification feed
        self._last_computes_refresh = datetime.datetime.now().timestamp()

        self._refreshingComputes = False
        # self._timer = QtCore.QTimer()
        # self._timer.setInterval(1000)
        # self._timer.timeout.connect(self._refreshComputesSlot)
        # self._timer.start()

    def _refreshComputesSlot(self):
        """
        Called when computes are refreshed.
        """

        if self._refreshingComputes:
            return
        if self._controller.connected() and datetime.datetime.now().timestamp() - self._last_computes_refresh > 1:
            self._last_computes_refresh = datetime.datetime.now().timestamp()
            self._refreshingComputes = True
            self._controller.get("/computes", self._listComputesCallback, show_progress=False, timeout=30)

    def _controllerConnectedSlot(self):
        """
        Called when connected to a compute.
        """

        if self._controller.connected():
            self._refreshingComputes = True
            self._controller.get("/computes", self._listComputesCallback, show_progress=False, timeout=30)

    def _controllerDisconnectedSlot(self):
        """
        Called when disconnected from the controller.
        """

        for compute_id in list(self._computes):
            del self._computes[compute_id]
            self.deleted_signal.emit(compute_id)

    def _listComputesCallback(self, result, error=False, **kwargs):
        """
        Callback to list computes.
        """

        self._refreshingComputes = False
        if error is True:
            log.error("Error while getting compute list: {}".format(result["message"]))
            return

        if result:
            for compute in result:
                self.computeDataReceivedCallback(compute)

    def computeDataReceivedCallback(self, compute):
        """
        Called when we received data from a compute node.
        """

        self._last_computes_refresh = datetime.datetime.now().timestamp()

        new_node = False
        compute_id = compute["compute_id"]
        if compute_id not in self._computes:
            new_node = True
            self._computes[compute_id] = Compute(compute_id)

        self._computes[compute_id].setName(compute["name"])
        self._computes[compute_id].setProtocol(compute["protocol"])
        self._computes[compute_id].setHost(compute["host"])
        self._computes[compute_id].setPort(compute["port"])
        self._computes[compute_id].setUser(compute["user"])
        if "connected" in compute:
            self._computes[compute_id].setConnected(compute["connected"])
            self._computes[compute_id].setCpuUsagePercent(compute["cpu_usage_percent"])
            self._computes[compute_id].setMemoryUsagePercent(compute["memory_usage_percent"])
            self._computes[compute_id].setDiskUsagePercent(compute["disk_usage_percent"])
            self._computes[compute_id].setCapabilities(compute["capabilities"])
            self._computes[compute_id].setLastError(compute.get("last_error"))

        if new_node:
            self.created_signal.emit(compute_id)
        else:
            self.updated_signal.emit(compute_id)

    def computeIsTheRemoteGNS3VM(self, compute):
        """
        :returns: boolean True if the remote server is the remote GNS3 VM
        """

        if compute.id() != "local" and compute.id() != "vm":
            if self.vmCompute() and "GNS3 VM ({})".format(compute.name()) == self.vmCompute().name():
                return True
        return False

    def computes(self):
        """
        :returns: List of computes nodes
        """

        computes = []
        for compute in self._computes.values():
            # We filter the remote GNS3 VM compute from the computes list
            if not self.computeIsTheRemoteGNS3VM(compute):
                computes.append(compute)
        return computes

    def vmCompute(self):
        """
        :returns: The GNS3 VM compute node or None
        """

        try:
            return self._computes["vm"]
        except KeyError:
            return None

    def localCompute(self):
        """
        :returns: The local compute node or None
        """

        try:
            return self._computes["local"]
        except KeyError:
            return None

    def localPlatform(self):
        """
        Return the platform of the local compute.

        With a remote controller it could be different of our local platform
        """

        c = self.localCompute()
        if c is None:
            return sys.platform
        return c.capabilities().get("platform", sys.platform)

    def remoteComputes(self):
        """
        :returns: List of non local and non VM computes
        """

        return [c for c in self._computes.values() if c.id() != "local" and c.id() != "vm"]

    def getCompute(self, compute_id):
        """
        Gets a compute by ID

        :param compute_id: compute identifier
        :returns: compute
        """

        if compute_id.startswith("http:") or compute_id.startswith("https:"):
            u = urllib.parse.urlsplit(compute_id)
            for compute in self._computes.values():
                if "{}:{}".format(compute.host(), compute.port()) == u.netloc:
                    return compute
            raise KeyError("Compute ID {} is missing.".format(compute_id))
        if compute_id not in self._computes:
            self._computes[compute_id] = Compute(compute_id)
            self.created_signal.emit(compute_id)
        return self._computes[compute_id]

    def connectToCompute(self, compute_id):
        """
        Connect to a compute
        """

        self._controller.post(f"/computes/{compute_id}/connect", callback=self._computeConnectCallback)

    def _computeConnectCallback(self, result, error=False, **kwargs):

        if error and "message" in result:
            from gns3.main_window import MainWindow
            parent = MainWindow.instance()
            QtWidgets.QMessageBox.critical(parent, "Remote compute", result.get("message"))

    def deleteCompute(self, compute_id):
        """
        Deletes a compute by ID

        :param compute_id: compute identifier
        """

        if compute_id in self._computes:
            del self._computes[compute_id]
            self._controller.delete("/computes/{compute_id}".format(compute_id=compute_id), None)
            self.deleted_signal.emit(compute_id)

    def updateList(self, computes):
        """
        Sync an array of compute with remote
        """

        for compute_id in copy.copy(self._computes):
            # Delete compute on controller not in the new computes
            if compute_id in ["local", "vm"]:
                continue

            if compute_id not in [c.id() for c in computes]:
                log.debug("Delete compute %s", compute_id)
                self.deleteCompute(compute_id)
            else:
                # Update the changed nodes
                for c in computes:
                    if c.id() == compute_id and c != self._computes[compute_id]:
                        log.debug("Update compute %s", compute_id)
                        self._controller.put("/computes/" + compute_id, None, body=c.__json__())
                        self._computes[compute_id] = c
                        self.updated_signal.emit(compute_id)
        # Create the new nodes
        for compute in computes:
            if compute.id() not in self._computes:
                log.debug("Create compute %s", compute.id())
                params = {"connect": True}
                self._controller.post("/computes", callback=self._computeCreatedCallback, body=compute.__json__(), params=params)
                self._computes[compute.id()] = compute
                self.created_signal.emit(compute.id())

    def _computeCreatedCallback(self, result, error=False, **kwargs):

        if error:
            log.error(result.get("message"))

    @staticmethod
    def reset():
        ComputeManager._instance = None

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of ComputeManager.
        :returns: instance of ComputeManager
        """

        if not hasattr(ComputeManager, '_instance') or ComputeManager._instance is None:
            ComputeManager._instance = ComputeManager()
        return ComputeManager._instance



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import hashlib
import tempfile
import json
import pathlib

from .qt import QtCore, QtGui, QtWebSockets, qpartial, qslot
from .symbol import Symbol
from .local_config import LocalConfig
from .settings import CONTROLLER_SETTINGS
from gns3.utils import parse_version
from gns3.http_client import HTTPClient

import logging
log = logging.getLogger(__name__)


class Controller(QtCore.QObject):
    """
    An instance of the server controller.
    """

    connected_signal = QtCore.Signal()
    disconnected_signal = QtCore.Signal()
    connection_failed_signal = QtCore.Signal()
    project_list_updated_signal = QtCore.Signal()
    image_list_updated_signal = QtCore.Signal()

    def __init__(self):

        super().__init__()
        self._connected = False
        self._connecting = False
        self._notification_stream = None
        self._version = None
        self._cache_directory = tempfile.TemporaryDirectory(suffix="-gns3")
        self._http_client = None
        self._first_error = True
        self._error_dialog = None
        self._display_error = True
        self._projects = []
        self._images = []
        self._websocket = QtWebSockets.QWebSocket()

        # If we do multiple call in order to download the same symbol we queue them
        self._static_asset_download_queue = {}

        self._loadSettings()

    def settings(self):
        """
        Returns the graphics view settings.

        :returns: settings dictionary
        """

        return self._settings

    def _loadSettings(self):
        """
        Loads the settings from the persistent settings file.
        """

        self._settings = LocalConfig.instance().loadSectionSettings(self.__class__.__name__, CONTROLLER_SETTINGS)

    def setSettings(self, new_settings):
        """
        Set new controller settings.

        :param new_settings: settings dictionary
        """

        # save the settings
        self._settings.update(new_settings)
        LocalConfig.instance().saveSectionSettings(self.__class__.__name__, self._settings)

    def host(self):

        return self._http_client.host()

    def version(self):

        return self._version

    def isRemote(self):
        """
        :returns Boolean: True if the controller is remote
        """

        return self._settings["remote"]

    def connecting(self):
        """
        :returns: True if connection is in progress
        """

        return self._connecting

    def connected(self):
        """
        Is the controller connected
        """

        return self._connected

    def httpClient(self):
        """
        :returns: HTTP client to connect to the controller
        """

        return self._http_client

    def setHttpClient(self, http_client):
        """
        :param http_client: Instance of HTTP client to communicate with the server
        """

        self._http_client = http_client
        if self._http_client:
            self._http_client.connected_signal.connect(self._httpClientConnectedSlot)
            self._http_client.disconnected_signal.connect(self._httpClientDisconnectedSlot)
            self._connectingToServer()

    def getHttpClient(self):
        """
        :return: Instance of HTTP client to communicate with the server
        """

        return self._http_client

    def setDisplayError(self, val):
        """
        Allow error to be visible or not
        """

        self._display_error = val
        self._first_error = True

    def connect(self):
        """
        Connect to controller
        """

        self._http_client = HTTPClient(self._settings)
        Controller.instance().setHttpClient(self._http_client)

    def _connectingToServer(self):
        """
        Connection process as started
        """

        self._connected = False
        self._connecting = True
        self.httpClient().connectToServer()

    def _httpClientDisconnectedSlot(self):

        if self._connected:
            self._connected = False
            self.disconnected_signal.emit()
            self._connectingToServer()
            self.stopListenNotifications()

    def _httpClientConnectedSlot(self):

        if not self._connected:
            self._connected = True
            self._connecting = False
            self.connected_signal.emit()
            self.refreshProjectList()
            self._startListenNotifications()

    def post(self, *args, **kwargs):
        return self.request("POST", *args, **kwargs)

    def get(self, *args, **kwargs):
        return self.request("GET", *args, **kwargs)

    def put(self, *args, **kwargs):
        return self.request("PUT", *args, **kwargs)

    def delete(self, *args, **kwargs):
        return self.request("DELETE", *args, **kwargs)

    def getCompute(self, path, compute_id, *args, **kwargs):
        """
        API get on a specific compute
        """

        compute_id = self.__fix_compute_id(compute_id)
        path = "/computes/{}{}".format(compute_id, path)
        return self.get(path, *args, **kwargs)

    def postCompute(self, path, compute_id, *args, **kwargs):
        """
        API post on a specific compute
        """

        compute_id = self.__fix_compute_id(compute_id)
        path = "/computes/{}{}".format(compute_id, path)
        return self.post(path, *args, **kwargs)

    def __fix_compute_id(self, compute_id):
        """
        Support for remote server <= 1.5
        This fix should be not require after the 2.1
        when all the templates will be managed on server
        """

        #FIXME: remove this?
        if compute_id.startswith("http:") or compute_id.startswith("https:"):
            from .compute_manager import ComputeManager
            try:
                return ComputeManager.instance().getCompute(compute_id).id()
            except KeyError:
                return compute_id
        return compute_id

    def request(self, method, path, *args, **kwargs):
        """
        Forward the query to the HTTP client or controller depending of the path
        """

        if self._http_client:
            return self._http_client.sendRequest(method, path, *args, **kwargs)

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of Controller.
        :returns: instance of Controller
        """

        if not hasattr(Controller, '_instance') or Controller._instance is None:
            Controller._instance = Controller()
        return Controller._instance

    def getStatic(self, url, callback, fallback=None):
        """
        Get a URL from the /static on controller and cache it on disk

        :param url: URL without the protocol and host part
        :param callback: Callback to call when file is ready
        :param fallback: Fallback url in case of error
        """

        if not self._http_client:
            return

        path = self.getStaticCachedPath(url)

        if os.path.exists(path):
            callback(path)
        elif path in self._static_asset_download_queue:
            self._static_asset_download_queue[path].append((callback, fallback, ))
        else:
            self._static_asset_download_queue[path] = [(callback, fallback, )]
            self._http_client.sendRequest("GET", url, qpartial(self._getStaticCallback, url, path), raw=True)

    def _getStaticCallback(self, url, path, result, error=False, **kwargs):

        if path not in self._static_asset_download_queue:
            return

        if error:
            fallback_used = False
            for callback, fallback in self._static_asset_download_queue[path]:
                if fallback:
                    self.getStatic(fallback, callback)
                fallback_used = True
            if fallback_used:
                log.debug("Error while downloading file: {}".format(url))
            del self._static_asset_download_queue[path]
            return
        try:
            with open(path, "wb+") as f:
                f.write(result)
        except OSError as e:
            log.error("Can't write to {}: {}".format(path, str(e)))
            return
        log.debug("File stored {} for {}".format(path, url))
        for callback, fallback in self._static_asset_download_queue[path]:
            callback(path)
        del self._static_asset_download_queue[path]

    def getStaticCachedPath(self, url):
        """
        Returns static cached (hashed) path

        :param url:
        """
        m = hashlib.md5()
        m.update(url.encode())
        if ".svg" in url:
            extension = ".svg"
        else:
            extension = ".png"
        path = os.path.join(self._cache_directory.name, m.hexdigest() + extension)
        return path

    def clearStaticCache(self):
        """
        Clear the cache directory.
        """

        for filename in os.listdir(self._cache_directory.name):
            if filename.endswith(".svg") or filename.endswith(".png"):
                try:
                    os.remove(os.path.join(self._cache_directory.name, filename))
                except OSError as e:
                    log.debug("Error deleting cached symbol '{}':{}".format(filename, e))
                    continue

    def getSymbolIcon(self, symbol_id, callback, fallback=None):
        """
        Get a QIcon for a symbol from the controller

        :param symbol_id: Symbol id
        :param callback: Callback to call when file is ready
        :param fallback: Fallback symbol if not found
        """
        if symbol_id is None:
            self.getStatic(Symbol(fallback).url(), qpartial(self._getIconCallback, callback))
        else:
            if fallback:
                fallback = Symbol(fallback).url()
            self.getStatic(Symbol(symbol_id).url(), qpartial(self._getIconCallback, callback), fallback=fallback)

    def _getIconCallback(self, callback, path):

        pixmap = QtGui.QPixmap(path)
        if pixmap.isNull():
            log.debug("Invalid symbol {}".format(path))
            path = ":/icons/cancel.svg"
        icon = QtGui.QIcon()
        icon.addFile(path)
        callback(icon)

    def uploadSymbol(self, symbol_id, path):

        self.post(
            "/symbols/" + symbol_id + "/raw",
            qpartial(self._finishSymbolUpload, path),
            body=pathlib.Path(path),
            progress_text="Uploading {}".format(symbol_id),
            timeout=None,
            wait=True
        )

    def _finishSymbolUpload(self, path, result, error=False, **kwargs):

        if error:
            log.error("Error while uploading symbol: {}: {}".format(path, result.get("message", "unknown")))
            return

        # Refresh the templates list
        from .template_manager import TemplateManager
        TemplateManager.instance().templates_changed_signal.emit()

    def getSymbols(self, callback):
        self.get('/symbols', callback=callback)

    def deleteProject(self, project_id, callback=None):
        Controller.instance().delete("/projects/{}".format(project_id), qpartial(self._deleteProjectCallback, callback=callback, project_id=project_id))

    def _deleteProjectCallback(self, result, error=False, project_id=None, callback=None, **kwargs):
        if error:
            log.error("Error while deleting project: {}".format(result["message"]))
        else:
            self.refreshProjectList()

        self._projects = [p for p in self._projects if p["project_id"] != project_id]

        if callback:
            callback(result, error=error, **kwargs)


    def createDiskImage(self, disk_name, options, callback):
        """
        Create a disk image on the controller

        :param callback: callback for the reply from the server
        """

        self.post(f"/images/qemu/{disk_name}", callback, body=options)


    @qslot
    def refreshProjectList(self, *args):
        self.get("/projects", self._projectListCallback)

    def _projectListCallback(self, result, error=False, **kwargs):
        if not error:
            self._projects = result
        self.project_list_updated_signal.emit()

    def projects(self):
        return self._projects

    @qslot
    def refreshImageList(self, *args):
        self.get("/images", self._imageListCallback)

    def _imageListCallback(self, result, error=False, **kwargs):
        if not error:
            self._images = result
        self.image_list_updated_signal.emit()

    def images(self):
        return self._images

    def _startListenNotifications(self):
        if not self.connected():
            return

        self._notification_stream = None

        # Qt websocket before Qt 5.6 doesn't support auth
        if parse_version(QtCore.QT_VERSION_STR) < parse_version("5.6.0") or parse_version(QtCore.PYQT_VERSION_STR) < parse_version("5.6.0"):
            self._notification_stream = Controller.instance().request(
                "GET",
                "/notifications",
                self._endListenNotificationCallback,
                download_progress_callback=self._event_received,
                timeout=None,
                show_progress=False
            )
        else:
            self._notification_stream = self._http_client.connectWebSocket(self._websocket, "/notifications/ws")
            self._notification_stream.textMessageReceived.connect(self._websocket_event_received)
            self._notification_stream.error.connect(self._websocket_error)
            self._notification_stream.sslErrors.connect(self._sslErrorsSlot)

    def stopListenNotifications(self):
        if self._notification_stream:
            log.debug("Stop listening for notifications from controller")
            stream = self._notification_stream
            self._notification_stream = None
            stream.abort()

    def _endListenNotificationCallback(self, result, error=False, **kwargs):
        """
        If notification stream disconnect we reconnect to it
        """
        if self._notification_stream:
            self._notification_stream = None
            self._startListenNotifications()

    @qslot
    def _websocket_error(self, error):
        if self._notification_stream:
            log.error("Websocket controller notification stream error: {}".format(self._notification_stream.errorString()))
            self.stopListenNotifications()

    @qslot
    def _sslErrorsSlot(self, ssl_errors):

        self._http_client.handleSslError(self._notification_stream, ssl_errors)

    @qslot
    def _websocket_event_received(self, event):
        try:
            self._event_received(json.loads(event))
        except ValueError as e:
            log.error("Invalid event received: {}".format(e))

    def _event_received(self, result, *args, **kwargs):

        # Log only relevant events
        if result["action"] not in ("ping", "compute.updated"):
            log.debug("Event received from controller stream: {}".format(result))
        if result["action"] == "template.created" or result["action"] == "template.updated":
            from gns3.template_manager import TemplateManager
            TemplateManager.instance().templateDataReceivedCallback(result["event"])
        elif result["action"] == "template.deleted":
            from gns3.template_manager import TemplateManager
            TemplateManager.instance().deleteTemplateCallback(result["event"])
        elif result["action"] == "compute.created" or result["action"] == "compute.updated":
            from .compute_manager import ComputeManager
            ComputeManager.instance().computeDataReceivedCallback(result["event"])
        elif result["action"] == "log.error":
            log.error(result["event"]["message"])
        elif result["action"] == "log.warning":
            log.warning(result["event"]["message"])
        elif result["action"] == "log.info":
            log.info(result["event"]["message"], extra={"show": True})
        elif result["action"] == "ping":
            pass



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/style_editor_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_StyleEditorDialog(object):
    def setupUi(self, StyleEditorDialog):
        StyleEditorDialog.setObjectName("StyleEditorDialog")
        StyleEditorDialog.resize(270, 294)
        StyleEditorDialog.setModal(True)
        self.verticalLayout = QtWidgets.QVBoxLayout(StyleEditorDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiStyleSettingsGroupBox = QtWidgets.QGroupBox(StyleEditorDialog)
        self.uiStyleSettingsGroupBox.setObjectName("uiStyleSettingsGroupBox")
        self.formLayout = QtWidgets.QFormLayout(self.uiStyleSettingsGroupBox)
        self.formLayout.setObjectName("formLayout")
        self.uiColorLabel = QtWidgets.QLabel(self.uiStyleSettingsGroupBox)
        self.uiColorLabel.setObjectName("uiColorLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.uiColorLabel)
        self.uiColorPushButton = QtWidgets.QPushButton(self.uiStyleSettingsGroupBox)
        self.uiColorPushButton.setText("")
        self.uiColorPushButton.setObjectName("uiColorPushButton")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.uiColorPushButton)
        self.uiBorderColorLabel = QtWidgets.QLabel(self.uiStyleSettingsGroupBox)
        self.uiBorderColorLabel.setObjectName("uiBorderColorLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.uiBorderColorLabel)
        self.uiBorderColorPushButton = QtWidgets.QPushButton(self.uiStyleSettingsGroupBox)
        self.uiBorderColorPushButton.setText("")
        self.uiBorderColorPushButton.setObjectName("uiBorderColorPushButton")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.uiBorderColorPushButton)
        self.uiBorderWidthLabel = QtWidgets.QLabel(self.uiStyleSettingsGroupBox)
        self.uiBorderWidthLabel.setObjectName("uiBorderWidthLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.uiBorderWidthLabel)
        self.uiBorderWidthSpinBox = QtWidgets.QSpinBox(self.uiStyleSettingsGroupBox)
        self.uiBorderWidthSpinBox.setMinimum(1)
        self.uiBorderWidthSpinBox.setMaximum(100)
        self.uiBorderWidthSpinBox.setProperty("value", 2)
        self.uiBorderWidthSpinBox.setObjectName("uiBorderWidthSpinBox")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.uiBorderWidthSpinBox)
        self.uiBorderStyleLabel = QtWidgets.QLabel(self.uiStyleSettingsGroupBox)
        self.uiBorderStyleLabel.setObjectName("uiBorderStyleLabel")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.uiBorderStyleLabel)
        self.uiBorderStyleComboBox = QtWidgets.QComboBox(self.uiStyleSettingsGroupBox)
        self.uiBorderStyleComboBox.setObjectName("uiBorderStyleComboBox")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.uiBorderStyleComboBox)
        self.uiRotationLabel = QtWidgets.QLabel(self.uiStyleSettingsGroupBox)
        self.uiRotationLabel.setObjectName("uiRotationLabel")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.uiRotationLabel)
        self.uiRotationSpinBox = QtWidgets.QSpinBox(self.uiStyleSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRotationSpinBox.sizePolicy().hasHeightForWidth())
        self.uiRotationSpinBox.setSizePolicy(sizePolicy)
        self.uiRotationSpinBox.setMinimum(-360)
        self.uiRotationSpinBox.setMaximum(360)
        self.uiRotationSpinBox.setObjectName("uiRotationSpinBox")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.uiRotationSpinBox)
        self.uiCornerRadiusLabel = QtWidgets.QLabel(self.uiStyleSettingsGroupBox)
        self.uiCornerRadiusLabel.setObjectName("uiCornerRadiusLabel")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.uiCornerRadiusLabel)
        self.uiCornerRadiusSpinBox = QtWidgets.QSpinBox(self.uiStyleSettingsGroupBox)
        self.uiCornerRadiusSpinBox.setMaximum(100)
        self.uiCornerRadiusSpinBox.setObjectName("uiCornerRadiusSpinBox")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.uiCornerRadiusSpinBox)
        self.verticalLayout.addWidget(self.uiStyleSettingsGroupBox)
        self.uiButtonBox = QtWidgets.QDialogButtonBox(StyleEditorDialog)
        self.uiButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.uiButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Apply|QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.uiButtonBox.setObjectName("uiButtonBox")
        self.verticalLayout.addWidget(self.uiButtonBox)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)

        self.retranslateUi(StyleEditorDialog)
        self.uiButtonBox.accepted.connect(StyleEditorDialog.accept) # type: ignore
        self.uiButtonBox.rejected.connect(StyleEditorDialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(StyleEditorDialog)

    def retranslateUi(self, StyleEditorDialog):
        _translate = QtCore.QCoreApplication.translate
        StyleEditorDialog.setWindowTitle(_translate("StyleEditorDialog", "Style editor"))
        self.uiStyleSettingsGroupBox.setTitle(_translate("StyleEditorDialog", "Style settings"))
        self.uiColorLabel.setText(_translate("StyleEditorDialog", "Fill color:"))
        self.uiBorderColorLabel.setText(_translate("StyleEditorDialog", "Border color:"))
        self.uiBorderWidthLabel.setText(_translate("StyleEditorDialog", "Border width:"))
        self.uiBorderWidthSpinBox.setSuffix(_translate("StyleEditorDialog", " px"))
        self.uiBorderStyleLabel.setText(_translate("StyleEditorDialog", "Border style:"))
        self.uiRotationLabel.setText(_translate("StyleEditorDialog", "Rotation:"))
        self.uiRotationSpinBox.setToolTip(_translate("StyleEditorDialog", "Rotation can be ajusted on the scene for a selected item while\n"
"editing (notes only) with ALT and \'+\' (or P) / ALT and \'-\' (or M)"))
        self.uiRotationSpinBox.setSuffix(_translate("StyleEditorDialog", "°"))
        self.uiCornerRadiusLabel.setText(_translate("StyleEditorDialog", "Corner radius:"))
        self.uiCornerRadiusSpinBox.setSuffix(_translate("StyleEditorDialog", "°"))
from . import resources_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/symbol_selection_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_SymbolSelectionDialog(object):
    def setupUi(self, SymbolSelectionDialog):
        SymbolSelectionDialog.setObjectName("SymbolSelectionDialog")
        SymbolSelectionDialog.resize(610, 700)
        SymbolSelectionDialog.setModal(True)
        self.verticalLayout = QtWidgets.QVBoxLayout(SymbolSelectionDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.uiCustomSymbolRadioButton = QtWidgets.QRadioButton(SymbolSelectionDialog)
        self.uiCustomSymbolRadioButton.setChecked(True)
        self.uiCustomSymbolRadioButton.setObjectName("uiCustomSymbolRadioButton")
        self.horizontalLayout_2.addWidget(self.uiCustomSymbolRadioButton)
        self.uiBuiltInSymbolRadioButton = QtWidgets.QRadioButton(SymbolSelectionDialog)
        self.uiBuiltInSymbolRadioButton.setObjectName("uiBuiltInSymbolRadioButton")
        self.horizontalLayout_2.addWidget(self.uiBuiltInSymbolRadioButton)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.uiCustomSymbolGroupBox = QtWidgets.QGroupBox(SymbolSelectionDialog)
        self.uiCustomSymbolGroupBox.setObjectName("uiCustomSymbolGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiCustomSymbolGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiSymbolToolButton = QtWidgets.QToolButton(self.uiCustomSymbolGroupBox)
        self.uiSymbolToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiSymbolToolButton.setObjectName("uiSymbolToolButton")
        self.gridLayout.addWidget(self.uiSymbolToolButton, 0, 2, 1, 1)
        self.uiSymbolLineEdit = QtWidgets.QLineEdit(self.uiCustomSymbolGroupBox)
        self.uiSymbolLineEdit.setObjectName("uiSymbolLineEdit")
        self.gridLayout.addWidget(self.uiSymbolLineEdit, 0, 1, 1, 1)
        self.uiSymbolLabel = QtWidgets.QLabel(self.uiCustomSymbolGroupBox)
        self.uiSymbolLabel.setObjectName("uiSymbolLabel")
        self.gridLayout.addWidget(self.uiSymbolLabel, 0, 0, 1, 1)
        self.verticalLayout.addWidget(self.uiCustomSymbolGroupBox)
        self.uiBuiltInGroupBox = QtWidgets.QGroupBox(SymbolSelectionDialog)
        self.uiBuiltInGroupBox.setObjectName("uiBuiltInGroupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiBuiltInGroupBox)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiSymbolTreeWidget = QtWidgets.QTreeWidget(self.uiBuiltInGroupBox)
        self.uiSymbolTreeWidget.setMinimumSize(QtCore.QSize(0, 300))
        self.uiSymbolTreeWidget.setIndentation(10)
        self.uiSymbolTreeWidget.setObjectName("uiSymbolTreeWidget")
        self.uiSymbolTreeWidget.headerItem().setText(0, "1")
        self.uiSymbolTreeWidget.header().setVisible(False)
        self.gridLayout_2.addWidget(self.uiSymbolTreeWidget, 1, 0, 1, 2)
        self.uiSearchLineEdit = QtWidgets.QLineEdit(self.uiBuiltInGroupBox)
        self.uiSearchLineEdit.setObjectName("uiSearchLineEdit")
        self.gridLayout_2.addWidget(self.uiSearchLineEdit, 0, 1, 1, 1)
        self.uiSearchLabel = QtWidgets.QLabel(self.uiBuiltInGroupBox)
        self.uiSearchLabel.setObjectName("uiSearchLabel")
        self.gridLayout_2.addWidget(self.uiSearchLabel, 0, 0, 1, 1)
        self.verticalLayout.addWidget(self.uiBuiltInGroupBox)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.uiButtonBox = QtWidgets.QDialogButtonBox(SymbolSelectionDialog)
        self.uiButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.uiButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Apply|QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.uiButtonBox.setObjectName("uiButtonBox")
        self.horizontalLayout.addWidget(self.uiButtonBox)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.uiBuiltInGroupBox.raise_()
        self.uiCustomSymbolGroupBox.raise_()

        self.retranslateUi(SymbolSelectionDialog)
        self.uiButtonBox.accepted.connect(SymbolSelectionDialog.accept)
        self.uiButtonBox.rejected.connect(SymbolSelectionDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(SymbolSelectionDialog)

    def retranslateUi(self, SymbolSelectionDialog):
        _translate = QtCore.QCoreApplication.translate
        SymbolSelectionDialog.setWindowTitle(_translate("SymbolSelectionDialog", "Symbol selection"))
        self.uiCustomSymbolRadioButton.setText(_translate("SymbolSelectionDialog", "Use a custom symbol"))
        self.uiBuiltInSymbolRadioButton.setText(_translate("SymbolSelectionDialog", "Symbols library"))
        self.uiCustomSymbolGroupBox.setTitle(_translate("SymbolSelectionDialog", "Custom symbol"))
        self.uiSymbolToolButton.setText(_translate("SymbolSelectionDialog", "&Browse..."))
        self.uiSymbolLabel.setText(_translate("SymbolSelectionDialog", "Path:"))
        self.uiBuiltInGroupBox.setTitle(_translate("SymbolSelectionDialog", "Symbols"))
        self.uiSearchLabel.setText(_translate("SymbolSelectionDialog", "Filter:"))

from . import resources_rc



#!/usr/bin/env python
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from ..compute_manager import ComputeManager
from ..qt import QtWidgets


def server_select(parent, node_type=None, allow_local_server=True):
    """
    Show a popup asking user to choose a server

    If only local server is available return it by default

    :params parent: Parent window
    :param node_type: Compute should support this node type (None allow all)
    :params allow_local_server: Boolean Is local server allowed
    :returns: Server or None
    """

    server_list = []

    server_priority = {}

    for compute in ComputeManager.instance().computes():
        if (compute.id() == "local" and allow_local_server) or compute.id() != "local":
            if node_type and node_type not in compute.capabilities().get("node_types", []):
                continue

            # We want local an VM first
            if compute.id() == "local":
                server_priority[compute.name()] = 0
            elif compute.id() == "vm":
                server_priority[compute.name()] = 5
            else:
                server_priority[compute.name()] = 10
            server_list.append(compute.name())

    # Alpha sort
    server_list = sorted(server_list)
    # Local and VM first
    server_list = sorted(server_list, key=lambda compute_name: server_priority[compute_name])

    if len(server_list) == 0:
        QtWidgets.QMessageBox.critical(parent, "Server", "No server available for this node type")
        return None
    elif len(server_list) == 1:
        selection = server_list[0]
    else:
        (selection, ok) = QtWidgets.QInputDialog.getItem(parent, "Server", "Please choose a server", server_list, 0, False)
        if not ok:
            return None

    for compute in ComputeManager.instance().computes():
        if selection == compute.name():
            return compute



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Nodes view that list all the available nodes to be dragged and dropped
on the QGraphics scene.
"""

from .qt import sip

from .qt import QtCore, QtGui, QtWidgets, qpartial
from .controller import Controller
from .template_manager import TemplateManager
from .dialogs.configuration_dialog import ConfigurationDialog
from .utils.get_icon import get_icon

from gns3.modules.builtin import Builtin
from gns3.modules.dynamips import Dynamips
from gns3.modules.iou import IOU
from gns3.modules.vpcs import VPCS
from gns3.modules.virtualbox import VirtualBox
from gns3.modules.qemu import Qemu
from gns3.modules.vmware import VMware
from gns3.modules.docker import Docker


import logging
log = logging.getLogger(__name__)


CATEGORY_TO_ID = {
    "firewall": 3,
    "guest": 2,
    "switch": 1,
    "multilayer_switch": 1,
    "router": 0
}

TEMPLATE_TYPE_TO_CONFIGURATION_PAGE = {
    "ethernet_switch": Builtin.configurationPage("ethernet_switch"),
    "ethernet_hub": Builtin.configurationPage("ethernet_hub"),
    "cloud": Builtin.configurationPage("cloud"),
    "dynamips": Dynamips.configurationPage(),
    "iou": IOU.configurationPage(),
    "vpcs": VPCS.configurationPage(),
    "virtualbox": VirtualBox.configurationPage(),
    "qemu": Qemu.configurationPage(),
    "vmware": VMware.configurationPage(),
    "docker": Docker.configurationPage()
}


class NodesView(QtWidgets.QTreeWidget):

    """
    Nodes view to list the nodes.

    :param parent: parent widget
    """

    def __init__(self, parent=None):

        super().__init__(parent)

        self._current_category = None
        self._current_search = ""

        # enables the possibility to drag items.
        self.setDragEnabled(True)
        TemplateManager.instance().templates_changed_signal.connect(self.refresh)

    def setCurrentSearch(self, search):
        self._current_search = search

    def refresh(self):

        self.clear()
        self.populateNodesView(self._current_category, self._current_search)

    def populateNodesView(self, category, search):
        """
        Populates the nodes view with the device list of the specified
        category (None = all devices).

        :param category: category of device to list
        :param search: filter
        """

        if not Controller.instance().connected():
            log.debug("Could not retrieve templates because there is no connection to the controller")
            return

        self.setIconSize(QtCore.QSize(32, 32))
        self._current_category = category
        self._current_search = search

        display_templates = set()
        for template in TemplateManager.instance().templates().values():
            if category is not None and category != template.category():
                continue
            if search != "" and search.lower() not in template.name().lower():
                continue

            display_templates.add(template.name())
            item = QtWidgets.QTreeWidgetItem(self)
            item.setText(0, template.name())
            item.setData(0, QtCore.Qt.UserRole, template.id())
            item.setData(1, QtCore.Qt.UserRole, "template")
            item.setSizeHint(0, QtCore.QSize(32, 32))
            Controller.instance().getSymbolIcon(template.symbol(),
                                                qpartial(self._setItemIcon, item),
                                                fallback=":/symbols/{}.svg".format(template.category()))

        self.sortByColumn(0, QtCore.Qt.AscendingOrder)

    def _setItemIcon(self, item, icon):
        if not sip.isdeleted(item):
            item.setIcon(0, icon)

    def _getMainWindow(self):
        from .main_window import MainWindow
        window = self.window()
        # when we're docked in main window
        if isinstance(window, MainWindow):
            return window
        # when we're in docked mode, outside main window
        return self.window().parent()

    def contextMenuEvent(self, event):
        """
        Handles all context menu events.

        :param event: QContextMenuEvent instance
        """

        self._showContextualMenu(event.globalPos())

    def mouseDoubleClickEvent(self, event):
        """
        Handles all mouse double click events.

        :param event: QMouseEvent instance
        """

        item = self.itemAt(event.pos())
        if item:
            template = TemplateManager.instance().getTemplate(item.data(0, QtCore.Qt.UserRole))
            if template:
                configuration_page = TEMPLATE_TYPE_TO_CONFIGURATION_PAGE.get(template.template_type())
                if not template.builtin() and configuration_page:
                    self._configurationSlot(template, configuration_page)
        super().mouseDoubleClickEvent(event)

    def mouseMoveEvent(self, event):
        """
        Handles all mouse move events.
        This is the starting point to drag & drop a template on the scene.

        :param: QMouseEvent instance
        """

        # Check that an item has been selected and left button clicked
        if self.currentItem() is not None and event.buttons() == QtCore.Qt.LeftButton:
            item = self.currentItem()
            icon = item.icon(0)
            mimedata = QtCore.QMimeData()

            assert item.data(1, QtCore.Qt.UserRole) == "template"
            template_id = item.data(0, QtCore.Qt.UserRole)

            mimedata.setData("application/x-gns3-template", template_id.encode())
            drag = QtGui.QDrag(self)
            drag.setMimeData(mimedata)
            drag.setPixmap(icon.pixmap(self.iconSize()))
            drag.setHotSpot(QtCore.QPoint(drag.pixmap().width(), drag.pixmap().height()))
            drag.exec_(QtCore.Qt.CopyAction)
            event.accept()

    def _showContextualMenu(self, pos):

        menu = QtWidgets.QMenu()
        refresh_action = QtWidgets.QAction("Refresh templates", menu)
        refresh_action.setIcon(get_icon("reload.svg"))
        refresh_action.triggered.connect(self.refresh)
        menu.addAction(refresh_action)

        item = self.currentItem()
        if item:
            template = TemplateManager.instance().getTemplate(item.data(0, QtCore.Qt.UserRole))
            if not template:
                return

            configuration_page = TEMPLATE_TYPE_TO_CONFIGURATION_PAGE.get(template.template_type())
            if not template.builtin() and configuration_page:
                configure_action = QtWidgets.QAction("Configure template", menu)
                configure_action.setIcon(get_icon("configuration.svg"))
                configure_action.triggered.connect(qpartial(self._configurationSlot, template, configuration_page))
                menu.addAction(configure_action)

                delete_action = QtWidgets.QAction("Delete template", menu)
                delete_action.setIcon(get_icon("delete.svg"))
                delete_action.triggered.connect(qpartial(self._deleteSlot, template))
                menu.addAction(delete_action)

        menu.exec_(pos)

    def _configurationSlot(self, template, configuration_page, source=None):

        dialog = ConfigurationDialog(template.name(), template.settings(), configuration_page(), parent=self)
        dialog.show()
        if dialog.exec_():
            TemplateManager.instance().updateTemplate(template)

    def _deleteSlot(self, template, source=None):

        msgbox = QtWidgets.QMessageBox(self)
        msgbox.setWindowTitle("Delete Template")
        msgbox.setText(f"Do you want to delete template '{template.name()}'?\n\n"
                       f"Deleting templates and images is irreversible!\n\n")
        msgbox.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        delete_all_button = QtWidgets.QPushButton(f"&Delete the template and orphaned images", msgbox)
        msgbox.addButton(delete_all_button, QtWidgets.QMessageBox.YesRole)
        delete_template_only_button = QtWidgets.QPushButton(f"&Only delete the template", msgbox)
        msgbox.addButton(delete_template_only_button, QtWidgets.QMessageBox.NoRole)
        abort_button = QtWidgets.QPushButton("&Cancel", msgbox)
        msgbox.addButton(abort_button, QtWidgets.QMessageBox.RejectRole)
        msgbox.setDefaultButton(abort_button)
        msgbox.setIcon(QtWidgets.QMessageBox.Question)
        msgbox.exec_()

        if msgbox.clickedButton() == delete_template_only_button:
            TemplateManager.instance().deleteTemplate(template.id())
        elif msgbox.clickedButton() == delete_all_button:
            TemplateManager.instance().deleteTemplate(template.id(), prune_images=True)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# __version__ is a human-readable version number.

# __version_info__ is a four-tuple for programmatic comparison. The first
# three numbers are the components of the version number. The fourth
# is zero for an official release, positive for a development branch,
# or negative for a release candidate or beta (after the base version
# number has been incremented)

__version__ = "3.0.0a5"
__version_info__ = (3, 0, 0, -99)

if "dev" in __version__:
    try:
        import os
        import subprocess
        if os.path.exists(os.path.join(os.path.dirname(os.path.abspath(__file__)), "..", ".git")):
            r = subprocess.check_output(["git", "rev-parse", "--short", "HEAD"]).decode().strip("\n")
            __version__ += "+" + r
    except Exception as e:
        print(e)
